@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -42,30 +37,25 @@
 
 #include "arla_local.h"
 
-RCSID("$Id: adir.c,v 1.58 1999/07/24 19:45:18 lha Exp $") ;
+RCSID("$Id: adir.c,v 1.66 2000/10/20 16:24:54 assar Exp $") ;
 
 /*
  *
  */
 
 static int
-get_fbuf_from_fid (VenusFid fid, CredCacheEntry *ce, fbuf *fbuf,
+get_fbuf_from_fid (VenusFid *fid, CredCacheEntry **ce,
+		   int *fd, fbuf *fbuf,
 		   FCacheEntry **centry,
 		   int open_flags, int fbuf_flags)
 {
     int ret;
 
-    ret = fcache_get (centry, fid, ce);
+    ret = fcache_get_data (centry, fid, ce);
     if (ret)
 	return ret;
 
-    ret = fcache_get_data (*centry, ce);
-    if (ret) {
-	fcache_release (*centry);
-	return ret;
-    }
-
-    ret = fcache_get_fbuf (*centry, fbuf, open_flags, fbuf_flags);
+    ret = fcache_get_fbuf (*centry, fd, fbuf, open_flags, fbuf_flags);
     if (ret) {
 	fcache_release(*centry);
 	return ret;
@@ -93,40 +83,57 @@ adir_lookup_fcacheentry (FCacheEntry *centry,
 			 CredCacheEntry *ce)
 {
      int ret;
+     int fd;
      fbuf the_fbuf;
 
-     ret = fcache_get_fbuf (centry, &the_fbuf, O_RDONLY, FBUF_READ);
+     ret = fcache_get_fbuf (centry, &fd, &the_fbuf, O_RDONLY,
+			    FBUF_READ|FBUF_PRIVATE);
      if (ret)
 	 return ret;
 
      ret = fdir_lookup (&the_fbuf, &dir, name, file);
      fbuf_end (&the_fbuf);
+     close (fd);
      return ret;
 }
 
 /*
- * Lookup `name' in the AFS directory identified by `dir' and return
- * the Fid in `file'.  All operations are done as `cred' and return
- * value is 0 or error code.
+ * Lookup ``name'' in the AFS directory identified by ``dir'' and
+ * return the Fid in ``file''.  All operations are done as ``cred''
+ * and return value is 0 or error code. If ``dentry'' is set, its the
+ * entry of the ``dir'' if returnvalue is 0.
  */
 
 int
-adir_lookup (VenusFid dir,
+adir_lookup (VenusFid *dir,
 	     const char *name,
 	     VenusFid *file,
-	     CredCacheEntry *ce)
+	     FCacheEntry **dentry, 
+	     CredCacheEntry **ce)
 {
+    int fd;
     fbuf the_fbuf;
     FCacheEntry *centry;
     int ret;
 
-    ret = get_fbuf_from_fid (dir, ce, &the_fbuf, &centry, O_RDONLY, FBUF_READ);
+    ret = get_fbuf_from_fid (dir, ce, &fd, &the_fbuf, &centry, O_RDONLY, 
+			     FBUF_READ|FBUF_PRIVATE);
     if (ret)
 	return ret;
 
-    ret = fdir_lookup (&the_fbuf, &dir, name, file);
+    ret = fdir_lookup (&the_fbuf, dir, name, file);
     fbuf_end (&the_fbuf);
-    fcache_release (centry);
+    close (fd);
+    if (dentry) {
+	if (ret) {
+	    *dentry = NULL;
+	    fcache_release (centry);
+	} else {
+	    *dentry = centry;
+	}
+    } else {
+	fcache_release (centry);
+    }
     return ret;
 }
 
@@ -136,21 +143,24 @@ adir_lookup (VenusFid dir,
  */
 
 int
-adir_changefid (VenusFid dir,
+adir_changefid (VenusFid *dir,
 		const char *name,
 		VenusFid *file,
-		CredCacheEntry *ce)
+		CredCacheEntry **ce)
 {
     FCacheEntry *centry;
     int ret;
+    int fd;
     fbuf the_fbuf;
 
-    ret = get_fbuf_from_fid (dir, ce, &the_fbuf, &centry, O_RDWR, FBUF_WRITE);
+    ret = get_fbuf_from_fid (dir, ce, &fd, &the_fbuf, &centry, O_RDWR, 
+			     FBUF_READ|FBUF_WRITE|FBUF_SHARED);
     if (ret)
 	return ret;
 
-    ret = fdir_changefid (&the_fbuf, name, file, &dir);
+    ret = fdir_changefid (&the_fbuf, name, file);
     fbuf_end (&the_fbuf);
+    close (fd);
     fcache_release (centry);
     return ret;
 }
@@ -160,20 +170,22 @@ adir_changefid (VenusFid dir,
  */
 
 int
-adir_emptyp (VenusFid dir,
-	     CredCacheEntry *ce)
+adir_emptyp (VenusFid *dir,
+	     CredCacheEntry **ce)
 {
      FCacheEntry *centry;
      int ret;
+     int fd;
      fbuf the_fbuf;
 
-     ret = get_fbuf_from_fid (dir, ce, &the_fbuf, &centry,
-			      O_RDONLY, FBUF_READ);
+     ret = get_fbuf_from_fid (dir, ce, &fd, &the_fbuf, &centry,
+			      O_RDONLY, FBUF_READ|FBUF_PRIVATE);
      if (ret)
 	 return ret;
 
      ret = fdir_emptyp (&the_fbuf);
      fbuf_end (&the_fbuf);
+     close (fd);
      fcache_release (centry);
      return ret;
 }
@@ -184,22 +196,24 @@ adir_emptyp (VenusFid dir,
  */
 
 int
-adir_readdir (VenusFid dir,
+adir_readdir (VenusFid *dir,
 	      void (*func)(VenusFid *, const char *, void *), 
 	      void *arg,
-	      CredCacheEntry *ce)
+	      CredCacheEntry **ce)
 {
+     int fd;
      fbuf the_fbuf;
      FCacheEntry *centry;
      int ret;
 
-     ret = get_fbuf_from_fid (dir, ce, &the_fbuf, &centry,
-			      O_RDONLY, FBUF_READ);
+     ret = get_fbuf_from_fid (dir, ce, &fd, &the_fbuf, &centry,
+			      O_RDONLY, FBUF_READ|FBUF_PRIVATE);
      if (ret)
 	 return ret;
 
-     ret = fdir_readdir (&the_fbuf, func, arg, &dir);
+     ret = fdir_readdir (&the_fbuf, func, arg, dir);
      fbuf_end (&the_fbuf);
+     close (fd);
      fcache_release (centry);
      return ret;
 }
@@ -213,18 +227,21 @@ adir_mkdir (FCacheEntry *dir,
 	    AFSFid dot,
 	    AFSFid dot_dot)
 {
+    int fd;
     fbuf the_fbuf;
     int ret;
 
-    assert (CheckLock(&dir->lock) == -1);
+    AssertExclLocked(&dir->lock);
 
-    ret = fcache_get_fbuf (dir, &the_fbuf, O_RDWR, FBUF_WRITE);
+    ret = fcache_get_fbuf (dir, &fd, &the_fbuf, O_RDWR,
+			   FBUF_READ|FBUF_WRITE|FBUF_SHARED);
     if (ret)
 	return ret;
 
     ret = fdir_mkdir (&the_fbuf, dot, dot_dot);
     fcache_update_length (dir, fbuf_len(&the_fbuf));
     fbuf_end (&the_fbuf);
+    close (fd);
     return ret;
 }
 
@@ -237,16 +254,19 @@ adir_creat (FCacheEntry *dir,
 	    const char *name,
 	    AFSFid fid)
 {
+    int fd;
     fbuf the_fbuf;
     int ret;
 
-    ret = fcache_get_fbuf (dir, &the_fbuf, O_RDWR, FBUF_WRITE);
+    ret = fcache_get_fbuf (dir, &fd, &the_fbuf, O_RDWR, 
+			   FBUF_READ|FBUF_WRITE|FBUF_SHARED);
     if (ret)
 	return ret;
 
     ret = fdir_creat (&the_fbuf, name, fid);
     fcache_update_length (dir, fbuf_len(&the_fbuf));
     fbuf_end (&the_fbuf);
+    close (fd);
     return ret;
 }
 
@@ -258,15 +278,18 @@ int
 adir_remove (FCacheEntry *dir,
 	     const char *name)
 {
+    int fd;
     fbuf the_fbuf;
     int ret;
 
-    ret = fcache_get_fbuf (dir, &the_fbuf, O_RDWR, FBUF_WRITE);
+    ret = fcache_get_fbuf (dir, &fd, &the_fbuf, O_RDWR, 
+			   FBUF_READ|FBUF_WRITE|FBUF_SHARED);
     if (ret)
 	return ret;
 
     ret = fdir_remove(&the_fbuf, name, NULL);
     fcache_update_length (dir, fbuf_len(&the_fbuf));
     fbuf_end (&the_fbuf);
+    close (fd);
     return ret;
 }
