@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,24 +35,13 @@
  * Check a directory in afs format.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-#include <stdio.h>
-#include <assert.h>
-#include <fcntl.h>
-#include <sys/types.h>
-#include <atypes.h>
-#include <sys/stat.h>
-#include <err.h>
-#include <roken.h>
+#include "arla_local.h"
+#include <getarg.h>
 
-#include "ko.h"
-#include "fbuf.h"
-#include "arladeb.h"
-#include "afs_dir.h"
+RCSID("$Id: afsdir_check.c,v 1.21 2000/10/10 16:27:06 lha Exp $");
 
-RCSID("$Id: afsdir_check.c,v 1.10 1999/08/02 02:22:07 assar Exp $");
+static int help_flag;
+static int verbose = 0;
 
 /*
  * Hash the filename of one entry.
@@ -95,7 +79,7 @@ getentry (DirPage0 *page0,
      return &page->entry[num % ENTRIESPERPAGE];
 }
 
-int
+static int
 check (const char *filename)
 {
     struct stat statbuf;
@@ -106,7 +90,7 @@ check (const char *filename)
     unsigned ind;
     unsigned len;
     int ret = 0;
-    unsigned npages;
+    unsigned npages = 0;
     unsigned page_entry_count;
     unsigned hash_entry_count;
     unsigned noverfill;
@@ -121,9 +105,15 @@ check (const char *filename)
 
     len = statbuf.st_size;
 
-    ret = fbuf_create (&the_fbuf, fd, len, FBUF_READ);
+    if (len == 0)
+	errx (1, "length == 0");
+
+    if (len % AFSDIR_PAGESIZE != 0)
+	errx (1, "length %% AFSDIR_PAGESIZE != 0");
+
+    ret = fbuf_create (&the_fbuf, fd, len, FBUF_READ|FBUF_PRIVATE);
     if (ret)
-	return ret;
+	err (1, "fbuf_create");
 
     page0 = (DirPage0 *)(the_fbuf.buf);
 
@@ -221,6 +211,12 @@ check (const char *filename)
 
 	    entry = getentry (page0, ind - 1);
 
+	    if (verbose)
+		printf ("%s - %ld.%ld\n",
+			entry->name, 
+			(long)ntohl(entry->fid.Vnode),
+			(long)ntohl(entry->fid.Unique));
+	    
 	    if (hashentry (entry->name) != i)
 		printf ("wrong name here? hash = %u, name = *%s*\n",
 			i, entry->name);
@@ -288,6 +284,7 @@ check (const char *filename)
 
 out:    
     fbuf_end (&the_fbuf);
+    close (fd);
     if (my_bitmaps) {
 	for (i = 0; i < npages; ++i)
 	    free (my_bitmaps[i]);
@@ -296,14 +293,36 @@ out:
     return ret;
 }
 
+static struct getargs args[] = {
+    {"verbose",	'v',	arg_flag,	&verbose,
+     "run in test mode", NULL},
+    {"help",	0,	arg_flag,	&help_flag,
+     NULL, NULL},
+};
+
+static void
+usage (int ret)
+{
+    arg_printusage (args, sizeof(args)/sizeof(*args), NULL, "[file]");
+    exit (ret);
+}
+
 int
 main(int argc, char **argv)
 {
-    if (argc != 2)
-	return 1;
+    int optind = 0;
+    
+    if (getarg (args, sizeof(args)/sizeof(*args), argc, argv, &optind))
+	usage (1);
 
-    arla_loginit ("/dev/stderr");
-    arla_log_set_level ("all");
+    argc -= optind;
+    argv += optind;
 
-    return check (argv[1]);
+    if (help_flag)
+	usage (0);
+
+    if (argc != 1)
+	usage (1);
+
+    return check (argv[0]);
 }
