@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -37,7 +32,7 @@
  */
 
 /*
- * $Id: arladeb.h,v 1.19 1999/04/20 20:58:07 map Exp $
+ * $Id: arladeb.h,v 1.24 2000/10/02 22:30:33 lha Exp $
  */
 
 #ifndef _arladeb_h
@@ -45,9 +40,14 @@
 
 #include <stdio.h>
 #include <stdarg.h>
+#include <log.h>
 
 #include <roken.h>
 
+extern Log_method* arla_log_method;
+extern Log_unit* arla_log_unit;
+extern struct units arla_deb_units[];
+
 /* masks */
 #define ADEBANY		0xffffffff
 #define ADEBMISC        0x00000001	/* misc debugging */
@@ -66,6 +66,8 @@
 #define ADEBERROR	0x10000000      /* don't ignore error */
 #define ADEBVLOG	0x20000000	/* venuslog output */
 
+#define ARLA_DEFAULT_LOG (ADEBWARN | ADEBERROR)
+
 extern struct units arla_deb_units[];
 
 void arla_log(unsigned level, char *fmt, ...);
@@ -120,4 +122,13 @@ arla_vwarnx (unsigned level, const char *fmt, va_list args)
 __attribute__ ((format (printf, 2, 0)))
 ;
 
+void
+arla_warnx_with_fid (unsigned level, const VenusFid *fid, const char *fmt, ...)
+__attribute__ ((format (printf, 3, 4)));
+
+void
+arla_vwarnx_with_fid (unsigned level, const VenusFid *fid, const char *fmt,
+		      va_list args)
+__attribute__ ((format (printf, 3, 0)));
+
 #endif				       /* _arladeb_h */
