@@ -33,7 +33,7 @@
 
 #include "arla_local.h"
 
-RCSID("$Id: reconnect.c,v 1.19 1999/07/24 22:24:02 lha Exp $");
+RCSID("$Id: reconnect.c,v 1.30 2000/10/20 16:24:55 assar Exp $");
 
 static int reconnect_nonmute(struct vcache *, int, struct timeval);
 static int reconnect_putattr(struct vcache *, struct xfs_attr *);
@@ -797,7 +797,7 @@ int reconnect_remove(struct vcache *vcp, FCacheEntry *childentry, char *name)
     AFSFetchStatus status;
     AFSVolSync volsync; 
     char tmp[2 * sizeof(int) + 2];
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
 
     fid = &(vcp->fid); /* points to the VenusFid structure */
     fid = fid_translate(fid);
@@ -822,7 +822,7 @@ int reconnect_remove(struct vcache *vcp, FCacheEntry *childentry, char *name)
   
 	if (isupdate)
 	{
-	    xfs_cache_handle cache_handle;
+	    fcache_cache_handle cache_handle;
 
 	    arla_log(ADEBDISCONN, 
 		     "reconnect_remove: can't remove because file modified!");
@@ -838,14 +838,20 @@ int reconnect_remove(struct vcache *vcp, FCacheEntry *childentry, char *name)
 
     res.res = 0;
 
-    assert (CheckLock(&fce->lock) == -1);
+    AssertExclLocked(&fce->lock);
 
     conn = find_first_fs (fce, ce, &context);
+    if (conn == NULL) {
+	arla_log (ADEBDISCONN, "find_first_fs failed", fce->index);
+	cred_free(ce);
+	ReleaseWriteLock(&fce->lock);
+	return ENETDOWN;
+    }
 
 /* Ba san: check the file exists 
 
    ReleaseWriteLock(&fce->lock);
-   error = adir_lookup(fce->fid, name, &tempfid, ce);
+   error = adir_lookup(fce->fid, name, &tempfid, NULL, &ce);
    assert (error == 0);                                             
    ObtainWriteLock(&fce->lock);  */
      
@@ -864,7 +870,7 @@ int reconnect_remove(struct vcache *vcp, FCacheEntry *childentry, char *name)
 
 /* Ba san: Chcek the deletion of the file */
     ReleaseWriteLock(&fce->lock);
-    error = adir_lookup(fce->fid, name, &tempfid, ce);
+    error = adir_lookup(&fce->fid, name, &tempfid, NULL, &ce);
     ObtainWriteLock(&fce->lock);
 
     if (error == 0) {
@@ -909,7 +915,7 @@ reconnect_rmdir(struct vcache *vcp, FCacheEntry *childEntry, char *name)
     char tmp[2 * sizeof(int) + 2];
     int ret = 0; 
     Result tempres; 
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
   
     ConnCacheEntry *conn;
     fs_server_context context;
@@ -925,9 +931,15 @@ reconnect_rmdir(struct vcache *vcp, FCacheEntry *childEntry, char *name)
     ce = cred_get (fid->Cell, vcp->cred.pag, CRED_ANY);
     assert (ce != NULL);
 
-    assert (CheckLock(&fce->lock) == -1);
+    AssertExclLocked(&fce->lock);
 
     conn = find_first_fs (fce, ce, &context);
+    if (conn == NULL) {
+	arla_log (ADEBDISCONN, "find_first_fs failed", fce->index);
+	cred_free(ce);
+	ReleaseWriteLock(&fce->lock);
+	return ENETDOWN;
+    }
 
     ret = RXAFS_RemoveDir (conn->connection,
 			   &fce->fid.fid,
@@ -943,7 +955,7 @@ reconnect_rmdir(struct vcache *vcp, FCacheEntry *childEntry, char *name)
 
 /* Ba san: Chcek the deletion of the file */
     ReleaseWriteLock(&fce->lock);
-    error = adir_lookup(fce->fid, name, &tempfid, ce);
+    error = adir_lookup(&fce->fid, name, &tempfid, NULL, &ce);
     ObtainWriteLock(&fce->lock);
 
     if (error == 0) {
@@ -989,10 +1001,14 @@ reconnect_mut_chk(FCacheEntry *fce, CredCacheEntry *ce, int version)
     AFSCallBack callback;                                                    
     AFSVolSync volsync;      
 
-    assert (CheckLock (&fetched.lock) == -1);
+    AssertExclLocked(&fetched.lock);
 
 /*SWW Aug 01: >= is changed into > */
     conn = find_first_fs (&fetched, ce, &context);
+    if (conn == NULL) {
+	arla_log (ADEBDISCONN, "find_first_fs failed", fce->index);
+	return ENETDOWN;
+    }
  
     ret = RXAFS_FetchStatus (conn->connection,
                              &fce->fid.fid,
@@ -1020,7 +1036,8 @@ reconnect_mut_chk(FCacheEntry *fce, CredCacheEntry *ce, int version)
  *
  */
 
-void fcache_backfile_name(char *name, size_t len)
+static void
+fcache_backfile_name(char *name, size_t len)
 {
     static int no = 1;
 
@@ -1033,7 +1050,7 @@ void fcache_backfile_name(char *name, size_t len)
  *
  */
 
-void
+static void
 copy_cached_file(int from, int to)
 {
     char name_from[2 * sizeof(int) + 1], name_to[2 * sizeof(int) + 1];
@@ -1064,18 +1081,18 @@ copy_cached_file(int from, int to)
  */
 
 static void
-update_fid (FCacheEntry *entry, VenusFid oldfid)
+reconnect_update_fid (FCacheEntry *entry, VenusFid oldfid)
 {
     if (entry->flags.kernelp)
-	update_kernelfid (oldfid, entry->fid);
+	update_fid (oldfid, NULL, entry->fid, entry);
 }
 
 /*
  *
  */
 
-int
-reconnect_mut_newfile(FCacheEntry **fcep, pag_t cred,VenusFid *new_fid)
+static int
+reconnect_mut_newfile(FCacheEntry **fcep, xfs_pag_t cred,VenusFid *new_fid)
 {
 
     FCacheEntry *parent_fce;
@@ -1088,7 +1105,7 @@ reconnect_mut_newfile(FCacheEntry **fcep, pag_t cred,VenusFid *new_fid)
     VenusFid newfid;
     int ret;
     int from, to;
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
 
     ret = fcache_find (&parent_fce, (*fcep)->parent);
     assert (ret == 0);
@@ -1150,7 +1167,6 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
     Result res;
     u_int32_t sizefs;
     AFSStoreStatus storestatus;
-    AFSCallBack callback;
     AFSVolSync volsync;
     int ret;
 
@@ -1193,6 +1209,7 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
 	alloc_fid_trans(fid);
 	reconnect_mut_newfile(&fce,vcp->cred.pag,&new_fid);  
 	fce->status.Length = sizefs;
+	fce->length = sizefs;
 	ReleaseWriteLock(&tempce->lock);
 	fill_fid_trans(&new_fid);
 	tempce->flags.attrp = FALSE;
@@ -1200,15 +1217,14 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
     }   
 #endif
 
-#if 0
-    usedbytes -= entry->status.Length;
-    usedbytes += size;
-    
-    fce->status.Length = size;  
-#endif
-
     /* code from truncate file XXX join */
     conn = find_first_fs (fce, ce, &context);
+    if (conn == NULL) {
+	arla_log (ADEBDISCONN, "find_first_fs failed.");
+	ReleaseWriteLock(&fce->lock);
+	cred_free(ce);
+	return ENETDOWN;
+    }
 
     if (fce->status.FileType != TYPE_DIR) {
 
@@ -1248,7 +1264,6 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
 
 	res.res = rx_EndCall (call, EndRXAFS_StoreData (call,
 							&status,
-							&callback,
 							&volsync));
 	if (res.res) {
 	    arla_log (ADEBDISCONN, "Error rx_EndCall: %s (%d)",
@@ -1257,7 +1272,6 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
 	}
 
 	fce->status   = status;
-	fce->callback = callback;
 	fce->volsync  = volsync;
 
 	volcache_update_volsync (fce->volume, fce->volsync);
@@ -1311,7 +1325,6 @@ reconnect_putdata(struct vcache *vcp)
     struct stat statinfo;
     AFSStoreStatus storestatus;
     AFSFetchStatus status;
-    AFSCallBack callback;
     AFSVolSync volsync;
     int ret;
 
@@ -1363,9 +1376,15 @@ reconnect_putdata(struct vcache *vcp)
 #endif
 
     /* code taken from write_data XXX join */ 
-    assert (CheckLock(&fce->lock) == -1);
+    AssertExclLocked(&fce->lock);
 
     conn = find_first_fs (fce, ce, &context);
+    if (conn == NULL) {
+	arla_log (ADEBDISCONN, "find_first_fs failed");
+	ReleaseWriteLock(&fce->lock); 
+	cred_free(ce);
+	return ENETDOWN;
+    }
 
     fd = fcache_open_file (fce, O_RDONLY);
     if (fd < 0) {
@@ -1411,7 +1430,6 @@ reconnect_putdata(struct vcache *vcp)
      
     res.res = rx_EndCall (call, EndRXAFS_StoreData (call,
 						    &status,
-						    &callback,
 						    &volsync));
     if (res.res) {
 	arla_log (ADEBDISCONN, "Error rx_EndCall: %s (%d)", 
@@ -1423,7 +1441,6 @@ reconnect_putdata(struct vcache *vcp)
 		 "reconnect: putdata, server incremented DataVersion!");
 
     fce->status   = status;
-    fce->callback = callback;
     fce->volsync  = volsync;
 
     volcache_update_volsync (fce->volume, fce->volsync);
@@ -1460,7 +1477,7 @@ int reconnect_rename(struct vcache *vcp_old, struct vcache *vcp_new,
     fs_server_context context;
     AFSFetchStatus orig_status, new_status;
     AFSVolSync volsync;
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
 
     fid_old = &vcp_old->fid;
     fid_old = fid_translate(fid_old);
@@ -1511,10 +1528,22 @@ int reconnect_rename(struct vcache *vcp_old, struct vcache *vcp_new,
     ce = cred_get (vcp_old->fid.Cell, vcp_old->cred.pag, CRED_ANY);
     assert (ce != NULL);
 
-    assert (CheckLock(&fce_old->lock) == -1
-            && CheckLock(&fce_new->lock) == -1);
+    AssertExclLocked(&fce_old->lock);
+    AssertExclLocked(&fce_new->lock);
 
     conn = find_first_fs (fce_old, ce, &context);
+    if (conn == NULL) {
+	arla_log (ADEBDISCONN, "find_first_fs failed");
+	ReleaseWriteLock(&fce_new->lock);
+	
+	if (fid_old->fid.Volume != fid_new->fid.Volume ||
+	    fid_old->fid.Vnode != fid_new->fid.Vnode   ||
+	    fid_old->fid.Unique != fid_new->fid.Unique )
+	    ReleaseWriteLock(&fce_old->lock); /* old and new are the same*/
+	
+	cred_free(ce);
+	return ENETDOWN;
+    }
 
     error = RXAFS_Rename (conn->connection,
 			  &fce_old->fid.fid,
@@ -1553,7 +1582,7 @@ int reconnect_rename(struct vcache *vcp_old, struct vcache *vcp_new,
 #endif
     ReleaseWriteLock(&fce_old->lock); /* old and new are the same*/
     
-    error = adir_lookup (fce_old->fid, name_old, &foo_fid, ce);
+    error = adir_lookup (&fce_old->fid, name_old, &foo_fid, NULL, &ce);
     
 #if 0
     if (fid_old->fid.Volume == fid_new->fid.Volume &&
@@ -1587,7 +1616,7 @@ int reconnect_rename(struct vcache *vcp_old, struct vcache *vcp_new,
 #endif
 	    ReleaseWriteLock(&fce_new->lock);
 	    
-	    error = adir_lookup (fce_new->fid, name_new, &foo_fid, ce);
+	    error = adir_lookup (&fce_new->fid, name_new, &foo_fid, NULL, &ce);
 	    
 #if 0
 	    if (fid_old->fid.Volume == fid_new->fid.Volume &&
@@ -1664,13 +1693,12 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
     AFSFetchStatus fetch_attr;
     AFSStoreStatus store_attr; 
 
-    Result res;
     AFSFetchStatus status;
     AFSCallBack callback;
     AFSVolSync volsync;
     int ret; 
     char tmp[2 * sizeof(int) + 2];
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
     int32_t type;
 
     parent_fid = &(parent->fid); /* points to the VenusFid structure */
@@ -1703,11 +1731,13 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
     adir_remove(parentEntry,name);  
 
     conn = find_first_fs (parentEntry, ce, &context);
-
     if (conn == NULL) {
-	arla_log (ADEBDISCONN, "Cannot call");
-	res.res = ENOMEM;
-	goto out;
+	arla_log (ADEBDISCONN, "find_first_fs failed");
+	ReleaseWriteLock(&parentEntry->lock);
+	ReleaseWriteLock(&childEntry->lock);
+	free_fs_server_context(&context);
+	cred_free(ce);
+	return ENETDOWN;
     }
 
     ret = fcache_find (&childEntry, *child_fid);
@@ -1755,7 +1785,7 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
 	    fill_fid_trans(&childEntry->fid);
 	    recon_hashtabadd(childEntry);
 	    childEntry->host = rx_HostOf (rx_PeerOf (conn->connection));
-	    update_fid (childEntry, fakeFid);
+	    reconnect_update_fid (childEntry, fakeFid);
 	} else {
 	    arla_log (ADEBDISCONN, "Could not CreateFile: %s (%d)",
 		      koerr_gettext(ret), ret);
@@ -1816,6 +1846,13 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
     childEntry->flags.datap = TRUE;
     childEntry->tokens |= XFS_ATTR_R | XFS_DATA_R | XFS_DATA_W;
 
+    if (parentEntry->volume == NULL)
+	ret = volcache_getbyid (parentEntry->fid.fid.Volume,
+				parentEntry->fid.Cell,
+				ce,
+				&parentEntry->volume,
+				&type);
+
     volcache_update_volsync (parentEntry->volume, parentEntry->volsync);
 
 
@@ -1836,7 +1873,7 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
 		childEntry->fid.fid.Vnode,
 		childEntry->fid.fid.Unique);
 
-    update_fid (childEntry, fakeFid);
+    reconnect_update_fid (childEntry, fakeFid);
 
     conv_dir(parentEntry, ce, 0, &cache_handle, tmp, sizeof(tmp));
 
@@ -1867,7 +1904,6 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
 
     FCacheEntry *parentEntry, *childEntry, *tempEntry, *tempparEntry;
     Result tempres;
-    Result res;
     int    ret = 0;
     int    tempret = 0;
     struct timeval tv;
@@ -1878,7 +1914,7 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
     AFSFetchStatus status;
     AFSCallBack  callback;
     AFSVolSync   volsync;
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
     int32_t type;
 
     parent_fid = &(parent->fid); /* points to the VenusFid structure */
@@ -1897,17 +1933,18 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
 
 /*Ba ba: used to check whether name can be find  Deleted !!!
   ReleaseWriteLock(&parentEntry->lock);
-  tempret = adir_lookup (parentEntry->fid , name , &foo_fid , ce);  */
+  tempret = adir_lookup (parentEntry->fid , name , &foo_fid , NULL, ce);  */
 /*Ba ba: used to check whether name can be find  Deleted !!! */
 
     /*Ba Wu Remove the dir name from itsparent dir */
     tempret = adir_remove(parentEntry,name);  
     conn = find_first_fs (parentEntry, ce, &context);
-
     if (conn == NULL) {
 	arla_log (ADEBDISCONN,"Cannot make this connection");
-	res.res = ENOMEM;
-	goto out;
+	ReleaseWriteLock(&parentEntry->lock);
+	ReleaseWriteLock(&childEntry->lock);
+	cred_free(ce);
+	return ENETDOWN;
     }
 
     ret = fcache_find(&childEntry, *child_fid);/*Ba Wu: remove the newly created dir */
@@ -1963,7 +2000,7 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
     ReleaseWriteLock(&tempparEntry->lock);
     tempret = adir_changefid (tempparEntry->fid ,name, &Outfid,  ce);
     ReleaseWriteLock(&tempparEntry->lock);
-    tempret = adir_lookup (tempparEntry->fid ,name, &foo_fid, ce);
+    tempret = adir_lookup (tempparEntry->fid ,name, &foo_fid, NULL, ce);
 #endif
 
     tempret = adir_creat (parentEntry, name, childEntry->fid.fid); 
@@ -1971,7 +2008,7 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
     childEntry->host = rx_HostOf (rx_PeerOf (conn->connection));
     assert(childEntry->host);
 
-    update_fid(childEntry, fakeFid);
+    reconnect_update_fid(childEntry, fakeFid);
      
     tempres = conv_dir(parentEntry, ce, 0, &cache_handle,
 		       tmp, sizeof(tmp));
@@ -2013,7 +2050,7 @@ int reconnect_link(struct vcache *parent, struct vcache *existing,
     AFSFetchStatus new_status;
     AFSFetchStatus status;
     AFSVolSync volsync;
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
 
 
     parent_fid = &(parent->fid);
@@ -2031,11 +2068,12 @@ int reconnect_link(struct vcache *parent, struct vcache *existing,
     assert (ret == 0);
 
     conn = find_first_fs (dir_entry, ce, &context);
-
     if (conn == NULL) {
 	arla_log (ADEBDISCONN,"Cannot make this connection");
-	res.res = ENOMEM;
-	goto out;
+	ReleaseWriteLock(&dir_entry->lock);
+	ReleaseWriteLock(&existing_entry->lock);
+	cred_free(ce);
+	return ENETDOWN;
     }
 
     ret = RXAFS_Link (conn->connection,
@@ -2062,6 +2100,7 @@ int reconnect_link(struct vcache *parent, struct vcache *existing,
  out:
     ReleaseWriteLock(&dir_entry->lock);
     ReleaseWriteLock(&existing_entry->lock);
+    cred_free(ce);
     free_fs_server_context (&context);
     return ret;
 }
@@ -2085,7 +2124,7 @@ int reconnect_symlink(struct vcache *parent, struct vcache *child,
 
     AFSFetchStatus fetch_attr, new_status;
     AFSVolSync volsync;
-    xfs_cache_handle cache_handle;
+    fcache_cache_handle cache_handle;
     int32_t type;
 
     parent_fid = &(parent->fid);
@@ -2108,10 +2147,12 @@ int reconnect_symlink(struct vcache *parent, struct vcache *child,
 
     assert(ret==0);
     conn = find_first_fs (dir_entry, ce, &context);
-
     if (conn == NULL) {
 	arla_log (ADEBDISCONN,"Cannot make this connection");
-	goto out;
+	ReleaseWriteLock(&dir_entry->lock);
+	ReleaseWriteLock(&childEntry->lock);
+	cred_free(ce);
+	return ENOMEM;
     }
   
     alloc_fid_trans(&childEntry->fid);
@@ -2152,7 +2193,7 @@ int reconnect_symlink(struct vcache *parent, struct vcache *child,
     childEntry->host = rx_HostOf (rx_PeerOf (conn->connection));
     assert(childEntry->host);
 
-    update_fid(childEntry, fakeFid);
+    reconnect_update_fid(childEntry, fakeFid);
     res = conv_dir (dir_entry, ce, 0, &cache_handle, tmp, sizeof(tmp));
 
  out: 
