@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,13 +35,15 @@
  * Our cache of volume information.
  */
 
-/* $Id: volcache.h,v 1.18 1999/07/28 00:34:09 assar Exp $ */
+/* $Id: volcache.h,v 1.28 2000/10/02 22:33:20 lha Exp $ */
 
 #ifndef _VOLCACHE_
 #define _VOLCACHE_
 
 #include <stdio.h>
 #include <cred.h>
+#include <list.h>
+#include "vldb.h"
 
 #define BACKSUFFIX ".backup"
 #define ROSUFFIX   ".readonly"
@@ -77,8 +74,13 @@ struct volcacheentry {
     nvldbentry entry;
     AFSVolSync volsync;
     int32_t cell;
-    unsigned refcount;
+    unsigned refcount;		/* number of files refererring this */
+    unsigned vol_refs;		/* number of volumes refing this */
     time_t last_fetch;
+    Listitem *li;
+    VenusFid mp_fid;		/* pointing to this volume */
+    VenusFid parent_fid;	/* .. of this volume */
+    struct volcacheentry *parent; /* parent volume */
     struct {
 	unsigned validp  : 1;
 	unsigned stablep : 1;
@@ -89,6 +91,20 @@ struct volcacheentry {
 
 typedef struct volcacheentry VolCacheEntry;
 
+/*
+ * This is magic cookie for the dump of the volcache.
+ * It's supposed not to be able to be confused with an old-style
+ * dump (with no header)
+ */
+
+#define VOLCACHE_MAGIC_COOKIE	0x00120100
+
+/*
+ * current version number of the dump file
+ */
+
+#define VOLCACHE_VERSION	0x2
+
 const char *volcache_get_rootvolume (void);
 
 void volcache_set_rootvolume (const char *volname);
@@ -99,29 +115,40 @@ int volcache_getbyname (const char *volname,
 			int32_t cell,
 			CredCacheEntry *ce,
 			VolCacheEntry **e,
-			int32_t *type);
+			int *type);
 
 int volcache_getbyid (u_int32_t id,
 		      int32_t cell,
 		      CredCacheEntry *ce,
 		      VolCacheEntry **e,
-		      int32_t *type);
+		      int *type);
 
 void volcache_update_volsync (VolCacheEntry *e, AFSVolSync volsync);
 
 void volcache_free (VolCacheEntry *e);
 
+void volcache_volref (VolCacheEntry *e, VolCacheEntry *parent);
+
+void volcache_volfree (VolCacheEntry *e);
+
 void volcache_invalidate (u_int32_t id, int32_t cell);
 
+void volcache_invalidate_ve (VolCacheEntry *ve);
+
 int volume_make_uptodate (VolCacheEntry *e, CredCacheEntry *ce);
 
 Bool volcache_reliable (u_int32_t id, int32_t cell);
 
+const char *volcache_getname (u_int32_t id, int32_t cell);
+
 void volcache_status (void);
 
 int
 volcache_store_state (void);
 
+int
+volcache_volid2bit (const VolCacheEntry *ve, u_int32_t volid);
+
 enum { VOLCACHE_OLD = 120 };
 
 #endif /* _VOLCACHE_ */
