@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,14 +31,16 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: ko.h,v 1.19 1999/07/06 17:29:28 lha Exp $ */
+/* $Id: ko.h,v 1.27 2000/10/08 07:44:38 lha Exp $ */
 
 #ifndef __KO_H
 #define __KO_H 1
 
+#include <sys/types.h>
 #include <netinet/in.h>
 #include <atypes.h>
 #include <bool.h>
+#include <log.h>
 
 typedef int32_t koerr_t;
 
@@ -65,22 +62,25 @@ const char *arla_getsysname(void);
  */
 
 typedef struct {
-     const char *name;
-     struct in_addr addr;
+    const char *name;
+    struct in_addr addr;
+    time_t timeout;		/* timeout of address */
 } cell_db_entry;
 
 typedef struct {
-    int32_t id;		/* Cell-ID */
+    int32_t id;			/* Cell-ID */
     const char *name;		/* Domain-style name */
     const char *expl;		/* Longer name */
     unsigned ndbservers;	/* # of database servers */
+    unsigned active_hosts;	/* # of active db servers */
     cell_db_entry *dbservers;	/* Database servers */
     enum { NOSUID_CELL, SUID_CELL } suid_cell ; /* if this is a suid cell */
+    time_t timeout;		/* when this entry expire */
 } cell_entry;
 
-void           cell_init (int cellcachesize);
+void	      cell_init (int cellcachesize, Log_method *log);
 
-const cell_db_entry *cell_dbservers (int32_t cell, int *);
+const cell_db_entry *cell_dbservers_by_id (int32_t cell, int *);
 
 const char    *cell_findnamedbbyname (const char *cell);
 const char    *cell_getthiscell (void);
@@ -90,6 +90,7 @@ const char    *cell_num2name (int32_t cell);
 cell_entry    *cell_get_by_name (const char *cellname);
 cell_entry    *cell_get_by_id (int32_t cell);
 cell_entry    *cell_new (const char *name);
+cell_entry    *cell_new_dynamic (const char *name);
 Bool           cell_issuid (cell_entry *c);
 Bool           cell_issuid_by_num (int32_t cell);
 Bool           cell_issuid_by_name (const char *cell);
@@ -97,7 +98,12 @@ Bool	       cell_setsuid_by_num (int32_t cell);
 int            cell_setthiscell (const char *cell);
 int	       cell_foreach (int (*func) (const cell_entry *, void *), 
 			     void *arg);
+const char    *cell_expand_cell (const char *cell);
 unsigned long  cell_get_version(void);
+Bool 	       cell_is_sanep (int cell);
+const char **  cell_thesecells (void);
+void 	       cell_print_cell (cell_entry *c, FILE *out);
+
 
 /*
  * misc vl
