@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -38,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls-common.c,v 1.37.2.1 1999/08/08 16:32:55 assar Exp $");
+RCSID("$Id: xfs_syscalls-common.c,v 1.59 2001/01/07 00:19:07 lha Exp $");
 
 /*
  * XFS system calls.
@@ -58,16 +53,33 @@ RCSID("$Id: xfs_syscalls-common.c,v 1.37.2.1 1999/08/08 16:32:55 assar Exp $");
 #elif defined(HAVE_SYS_IOCTL_H)
 #include <sys/ioctl.h>
 #endif
+/*
+ * XXX - horrible kludge. If we're built without HAVE_CONFIG_H we assume that
+ *       we're built inside the kernel on OpenBSD.
+ */
+#ifdef HAVE_CONFIG_H
 #include <kafs.h>
+#else
+#include <xfs/xfs_pioctl.h>
+#endif
 
 /*
  * the syscall entry point
  */
 
+#if defined(_LKM) || defined(KLD_MODULE) || defined(__osf__) || defined(__APPLE__)
 int
 xfspioctl(struct proc *proc, void *varg, register_t *return_value)
+#else
+int
+sys_xfspioctl(struct proc *proc, void *varg, register_t *return_value)
+#endif
 {
+#if defined(_LKM) || defined(KLD_MODULE) || defined(__osf__) || defined(__APPLE__)
     struct sys_pioctl_args *arg = (struct sys_pioctl_args *) varg;
+#else
+    struct sys_xfspioctl_args *arg = (struct sys_xfspioctl_args *) varg;
+#endif
     int error = EINVAL;
 
     switch (SCARG(arg, operation)) {
@@ -124,7 +136,7 @@ xfs_is_pag(struct ucred *cred)
  * Return the pag used by `cred'
  */
 
-pag_t
+xfs_pag_t
 xfs_get_pag(struct ucred *cred)
 {
     if (xfs_is_pag(cred)) {
@@ -187,6 +199,7 @@ xfs_setpag_call(struct ucred **ret_cred)
     return 0;
 }
 
+#if defined(_LKM) || defined(KLD_MODULE) || defined(__osf__) || defined(__APPLE__)
 /*
  * remove a pag
  */
@@ -240,6 +253,7 @@ xfs_setgroups (struct proc *p,
 	return ret;
     }
 }
+#endif
 
 /*
  * Return the vnode corresponding to `pathptr'
@@ -259,14 +273,14 @@ lookup_node (const char *pathptr,
     struct nameidata nd, *ndp = &nd;
 #endif
     struct vnode *vp;
-    size_t done;
+    size_t count;
 
-    XFSDEB(XDEBSYS, ("xfs_syscall: looking up: %p\n", pathptr));
+    XFSDEB(XDEBSYS, ("xfs_syscall: looking up: %lx\n",
+		     (unsigned long)pathptr));
 
-    error = copyinstr(pathptr, path, MAXPATHLEN, &done);
+    error = copyinstr((char *) pathptr, path, MAXPATHLEN, &count);
 
-    XFSDEB(XDEBSYS, ("xfs_syscall: looking up: %s len: %lu error: %d\n", 
-		     path, (unsigned long)done, error));
+    XFSDEB(XDEBSYS, ("xfs_syscall: looking up: %s, error: %d\n", path, error));
 
     if (error)
 	return error;
@@ -289,11 +303,41 @@ lookup_node (const char *pathptr,
 }
 
 /*
- * return file handle of `vp' in vice_ioctl->out
+ * implement xfs fhget in a way that should be compatible with the native
+ * getfh
  */
 
 static int
-fhget_call (struct proc *p,
+getfh_compat (struct proc *p,
+	      struct ViceIoctl *vice_ioctl,
+	      struct vnode *vp)
+{
+    /* This is to be same as getfh */
+    fhandle_t fh;
+    int error;
+	
+    bzero((caddr_t)&fh, sizeof(fh));
+    fh.fh_fsid = vp->v_mount->mnt_stat.f_fsid;
+#if __osf__
+    VFS_VPTOFH(vp, &fh.fh_fid, error);
+#else
+    error = VFS_VPTOFH(vp, &fh.fh_fid);
+#endif
+    if (error)
+	return error;
+
+    if (vice_ioctl->out_size < sizeof(fh))
+	return EINVAL;
+	
+    return copyout((caddr_t)&fh, vice_ioctl->out, sizeof (fh));
+}
+
+/*
+ * implement xfs fhget by combining (dev, ino, generation)
+ */
+
+static int
+trad_fhget (struct proc *p,
 	    struct ViceIoctl *vice_ioctl,
 	    struct vnode *vp)
 {
@@ -304,36 +348,9 @@ fhget_call (struct proc *p,
     struct xfs_fhandle_t xfs_handle;
     struct xfs_fh_args fh_args;
 
-    XFSDEB(XDEBSYS, ("fhget_call\n"));
-
-    if (vp == NULL)
-	return EBADF;
-
-    error = xfs_suser (p);
-    if (error)
-	return error;
-
-#if (defined(HAVE_GETFH) && defined(HAVE_FHOPEN)) || defined(__osf__)
-    {
-	/* This is to be same as getfh */
-	fhandle_t fh;
-	
-	memset((caddr_t)&fh, 0, sizeof(fh));
-	fh.fh_fsid = vp->v_mount->mnt_stat.f_fsid;
-        error = VFS_VPTOFH(vp, &fh.fh_fid);
-        if (error)
-	    return (error);
-
-	if (vice_ioctl->out_size < sizeof(fh))
-	    return EINVAL;
-	
-	error = copyout((caddr_t)&fh, vice_ioctl->out, sizeof (fh));
-	return (error);
-    }
-#else
     xfs_vop_getattr(vp, &vattr, xfs_proc_to_cred(p), p, error);
     if (error)
-	goto out;
+	return error;
 
     mnt = vp->v_mount;
 
@@ -345,21 +362,50 @@ fhget_call (struct proc *p,
     memcpy (xfs_handle.fhdata, &fh_args, sizeof(fh_args));
     len = sizeof(xfs_handle);
 
-    if (vice_ioctl->out_size < len) {
-	error = EINVAL;
-	goto out;
-    }
+    if (vice_ioctl->out_size < len)
+	return EINVAL;
 
     error = copyout (&xfs_handle, vice_ioctl->out, len);
     if (error) {
 	XFSDEB(XDEBSYS, ("fhget_call: copyout failed: %d\n", error));
     }
-    
- out:
-    vrele (vp);
     return error;
+}
+
+/*
+ * return file handle of `vp' in vice_ioctl->out
+ * vp is vrele:d
+ */
+
+static int
+fhget_call (struct proc *p,
+	    struct ViceIoctl *vice_ioctl,
+	    struct vnode *vp)
+{
+    int error;
+
+    XFSDEB(XDEBSYS, ("fhget_call\n"));
+
+    if (vp == NULL)
+	return EBADF;
+
+#ifdef __APPLE__
+    error = EINVAL; /* XXX: Leaks vnodes if fhget/fhopen is used */
+    goto out;
+#endif
+
+    error = xfs_suser (p);
+    if (error)
+	goto out;
+
+#if (defined(HAVE_GETFH) && defined(HAVE_FHOPEN)) || defined(__osf__)
+    error = getfh_compat (p, vice_ioctl, vp);
+#else
+    error = trad_fhget (p, vice_ioctl, vp);
 #endif /* HAVE_GETFH && HAVE_FHOPEN */
-    
+out:
+    vrele(vp);
+    return error;
 }
 
 /*
@@ -373,8 +419,6 @@ fhopen_call (struct proc *p,
 	     int flags,
 	     register_t *retval)
 {
-    int error;
-    struct xfs_fhandle_t fh;
 
     XFSDEB(XDEBSYS, ("fhopen_call: flags = %d\n", flags));
 
@@ -383,17 +427,12 @@ fhopen_call (struct proc *p,
 	return EINVAL;
     }
 
-    if (vice_ioctl->in_size < sizeof(fh))
-	return EINVAL;
-
-    error = copyin (vice_ioctl->in,
-		    &fh,
-		    sizeof(fh));
-    if (error)
-	return error;
+#ifdef __APPLE__
+    return EINVAL; /* XXX: Leaks vnodes if fhget/fhopen is used */
+#endif
 
     return xfs_fhopen (p,
-		       &fh,
+		       (struct xfs_fhandle_t *)vice_ioctl->in,
 		       flags,
 		       retval);
 }
@@ -449,13 +488,13 @@ remote_pioctl (struct proc *p,
     msg.cred.uid = xfs_proc_to_ruid(p);
     msg.cred.pag = xfs_get_pag(xfs_proc_to_cred(p));
 
-    error = xfs_message_rpc(0, &msg.header, sizeof(msg)); /* XXX */
+    error = xfs_message_rpc(0, &msg.header, sizeof(msg), p); /* XXX */
     msg2 = (struct xfs_message_wakeup_data *) &msg;
 
     if (error == 0)
 	error = msg2->error;
-    else
-	error = EINVAL; /* return EINVAL to not confuse applications */
+    if (error == ENODEV)
+	error = EINVAL;
 
     if (error == 0 && msg2->header.opcode == XFS_MSG_WAKEUP_DATA)
 	error = copyout(msg2->msg, vice_ioctl->out, 
@@ -516,11 +555,11 @@ xfs_pioctl_call(struct proc *proc,
     char *pathptr;
     struct vnode *vp = NULL;
 
-    XFSDEB(XDEBSYS, ("xfs_syscall(%d, %p, %d, %p, %d)\n", 
+    XFSDEB(XDEBSYS, ("xfs_syscall(%d, %lx, %d, %lx, %d)\n", 
 		     SCARG(arg, operation),
-		     SCARG(arg, a_pathP),
+		     (unsigned long)SCARG(arg, a_pathP),
 		     SCARG(arg, a_opcode),
-		     SCARG(arg, a_paramsP),
+		     (unsigned long)SCARG(arg, a_paramsP),
 		     SCARG(arg, a_followSymlinks)));
 
     /* Copy in the data structure for us */
@@ -547,7 +586,9 @@ xfs_pioctl_call(struct proc *proc,
     case VIOC_FHOPEN :
 	return fhopen_call (proc, &vice_ioctl, vp,
 			    SCARG(arg, a_followSymlinks), return_value);
-    case VIOC_XFSDEBUG:
+    case VIOC_XFSDEBUG :
+	if (vp != NULL)
+	    vrele (vp);
 	return xfs_debug (proc, &vice_ioctl);
     default :
 	XFSDEB(XDEBSYS, ("a_opcode = %x\n", SCARG(arg, a_opcode)));
