@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -38,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_vfsops-osf.c,v 1.16 1999/08/01 10:35:25 lha Exp $");
+RCSID("$Id: xfs_vfsops-osf.c,v 1.22 2000/10/02 23:33:23 lha Exp $");
 
 /*
  * XFS vfs operations.
@@ -64,21 +59,23 @@ xfs_mount(struct mount * mp,
 static int
 xfs_start(struct mount * mp, int flags)
 {
-    XFSDEB(XDEBVFOPS, ("xfs_start mp = %p, flags = %d\n", mp, flags));
+    XFSDEB(XDEBVFOPS, ("xfs_start mp = %lx, flags = %d\n",
+		       (unsigned long)mp, flags));
     return 0;
 }
 
 static int
 xfs_unmount(struct mount * mp, int mntflags)
 {
-    XFSDEB(XDEBVFOPS, ("xfs_umount: mp = %p, mntflags = %d\n", mp, mntflags));
+    XFSDEB(XDEBVFOPS, ("xfs_umount: mp = %lx, mntflags = %d\n",
+		       (unsigned long)mp, mntflags));
     return xfs_unmount_common(mp, mntflags);
 }
 
 static int
 xfs_root(struct mount * mp, struct vnode ** vpp)
 {
-    XFSDEB(XDEBVFOPS, ("xfs_root mp = %p\n", mp));
+    XFSDEB(XDEBVFOPS, ("xfs_root mp = %lx\n", (unsigned long)mp));
 
     return xfs_root_common (mp, vpp, u.u_procp, u.u_procp->p_rcred);
 }
@@ -86,24 +83,26 @@ xfs_root(struct mount * mp, struct vnode ** vpp)
 static int
 xfs_quotactl(struct mount *mp, int cmd, uid_t uid, caddr_t arg)
 {
-    XFSDEB(XDEBVFOPS, ("xfs_quotactl: mp = %p, cmd = %d, uid = %u, "
-		       "arg = %p\n", 
-		       mp, cmd, uid, arg));
+    XFSDEB(XDEBVFOPS, ("xfs_quotactl: mp = %lx, cmd = %d, uid = %u, "
+		       "arg = %lx\n", 
+		       (unsigned long)mp, cmd, uid,
+		       (unsigned long)arg));
     return (EOPNOTSUPP);
 }
 
 static int
 xfs_statfs(struct mount *mp)
 {
-    XFSDEB(XDEBVFOPS, ("xfs_statfs: mp = %p\n", mp));
+    XFSDEB(XDEBVFOPS, ("xfs_statfs: mp = %lx\n",
+		       (unsigned long)mp));
     return 0;
 }
 
 static int
 xfs_sync(struct mount *mp, int waitfor)
 {
-    XFSDEB(XDEBVFOPS, ("xfs_sync: mp = %p, waitfor = %d\n",
-		       mp, waitfor));
+    XFSDEB(XDEBVFOPS, ("xfs_sync: mp = %lx, waitfor = %d\n",
+		       (unsigned long)mp, waitfor));
     return 0;
 }
 
@@ -141,7 +140,8 @@ make_dead_vnode(struct mount *mp, struct vnode **vpp)
 {
     int error;
 
-    XFSDEB(XDEBNODE, ("make_dead_vnode mp = %p\n", mp));
+    XFSDEB(XDEBNODE, ("make_dead_vnode mp = %lx\n",
+		      (unsigned long)mp));
 
     error = getnewvnode(VT_NON, &dead_vnodeops, vpp);
     if (error) {
@@ -176,7 +176,7 @@ struct vfsops xfs_vfsops = {
 extern struct vfsops xfs_vfsops;
 
 int
-xfs_install_fs()
+xfs_install_fs(void)
 {
     if (vn_maxprivate < sizeof(struct xfs_node))
 	return ENOMEM;
@@ -184,14 +184,14 @@ xfs_install_fs()
 }
 
 int
-xfs_uninstall_fs()
+xfs_uninstall_fs(void)
 {
     return vfssw_del(MOUNT_XFS);
 }
 
 int
 xfs_fhlookup (struct proc *proc,
-	      xfs_fhandle_t *fhp,
+	      struct xfs_fhandle_t *fhp,
 	      struct vnode **vpp)
 {
     int error;
@@ -200,8 +200,7 @@ xfs_fhlookup (struct proc *proc,
     fid_t fid;
     fhandle_t *fh = (fhandle_t *)fhp;
 
-    XFSDEB(XDEBVFOPS, ("xfs_fhlookup: %ld.%ld.%ld\n",
-		       fsid, fileid, gen));
+    XFSDEB(XDEBVFOPS, ("xfs_fhlookup\n"));
 
     error = xfs_suser (proc);
     if (error)
@@ -239,7 +238,7 @@ xfs_fhlookup (struct proc *proc,
 
 int
 xfs_fhopen (struct proc *proc,
-	    xfs_fhandle_t *fhp,
+	    struct xfs_fhandle_t *fhp,
 	    int user_flags,
 	    register_t *retval)
 {
@@ -251,9 +250,11 @@ xfs_fhopen (struct proc *proc,
     struct file *fp;
     extern struct fileops vnops;
     struct ufile_state *ufp;
+#ifndef U_FD_SET
+    struct ufile_entry *fe;
+#endif
 
-    XFSDEB(XDEBVFOPS, ("xfs_fhopen: fileid = %ld, flags = %x\n",
-		       fileid, user_flags));
+    XFSDEB(XDEBVFOPS, ("xfs_fhopen: flags = %x\n", user_flags));
 
     error = xfs_fhlookup (proc, fhp, &vp);
     if (error)
@@ -267,12 +268,20 @@ xfs_fhopen (struct proc *proc,
     if(error)
 	goto out;
 
+#ifdef U_FD_SET
     error = falloc(&fp, &index);
+#else
+    error = falloc(&fp, &index, &fe);
+#endif
     if (error)
 	goto out;
     ufp = &u.u_file_state;
     FP_UNREF_MT(fp,ufp);
+#ifdef U_FD_SET
     U_FD_SET(index, fp, ufp);
+#else
+    u_set_fe(index, fe, fp, ufp);
+#endif
 
     /* ??? */
     if (flags & FREAD) {
