@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -40,6 +35,10 @@
  * XFS operations.
  */
 
+#ifdef __APPLE__
+#define MACH_KERNEL 1
+#endif
+
 #include <xfs/xfs_locl.h>
 #include <xfs/xfs_message.h>
 #include <xfs/xfs_common.h>
@@ -48,12 +47,11 @@
 #include <xfs/xfs_deb.h>
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_vnodeops.h>
-#include <vm/vm.h>
 #ifdef HAVE_VM_VNODE_PAGER_H
 #include <vm/vnode_pager.h>
 #endif
 
-RCSID("$Id: xfs_vnodeops-bsd.c,v 1.53 1999/08/02 02:19:53 assar Exp $");
+RCSID("$Id: xfs_vnodeops-bsd.c,v 1.87 2001/01/07 00:18:46 lha Exp $");
 
 /*
  * vnode functions
@@ -70,12 +68,7 @@ xfs_open(struct vop_open_args * ap)
           struct proc *p;
   }; */
 {
-    XFSDEB(XDEBVNOPS, ("xfs_open\n"));
-
-    if (ap->a_mode & FWRITE)
-	return xfs_open_valid(ap->a_vp, ap->a_cred, XFS_OPEN_NW);
-    else
-	return xfs_open_valid(ap->a_vp, ap->a_cred, XFS_OPEN_NR);
+    return xfs_open_common (ap->a_vp, ap->a_mode, ap->a_cred, ap->a_p);
 }
 #endif /* HAVE_VOP_OPEN */
 
@@ -205,22 +198,23 @@ static int
 xfs_getattr(struct vop_getattr_args * ap)
      /* struct vnode *vp,
 	    struct vattr *vap,
-	    struct ucred *cred) */
+	    struct ucred *cred,
+	    struct proc *p) */
 {
-    return xfs_getattr_common(ap->a_vp, ap->a_vap, ap->a_cred);
+    return xfs_getattr_common(ap->a_vp, ap->a_vap, ap->a_cred, ap->a_p);
 }
 #endif /* HAVE_VOP_GETATTR */
 
 #ifdef HAVE_VOP_SETATTR
 static int
 xfs_setattr(struct vop_setattr_args * ap)
-     /*
-struct vnode *vp,
+     /* struct vnode *vp,
 	    struct vattr *vap,
-	    struct ucred *cred)
+	    struct ucred *cred,
+	    struct proc *p)
 	    */
 {
-    return xfs_setattr_common(ap->a_vp, ap->a_vap, ap->a_cred);
+    return xfs_setattr_common(ap->a_vp, ap->a_vap, ap->a_cred, ap->a_p);
 }
 #endif /* HAVE_VOP_SETATTR */
 
@@ -230,10 +224,11 @@ xfs_access(struct vop_access_args * ap)
      /*
 struct vnode *vp,
 	   int mode,
-	   struct ucred *cred)
+	   struct ucred *cred,
+	   struct proc *p)
 	   */
 {
-    return xfs_access_common(ap->a_vp, ap->a_mode, ap->a_cred);
+    return xfs_access_common(ap->a_vp, ap->a_mode, ap->a_cred, ap->a_p);
 }
 #endif /* HAVE_VOP_ACCESS */
 
@@ -249,6 +244,8 @@ xfs_lookup(struct vop_lookup_args * ap)
 {
     struct componentname *cnp = ap->a_cnp;
     int error;
+    int lockparent = (cnp->cn_flags & (LOCKPARENT | ISLASTCN))
+	== (LOCKPARENT | ISLASTCN);
 
     XFSDEB(XDEBVNOPS, ("xfs_lookup: (%s, %ld), nameiop = %lu, flags = %lu\n",
 		       cnp->cn_nameptr,
@@ -256,6 +253,10 @@ xfs_lookup(struct vop_lookup_args * ap)
 		       cnp->cn_nameiop,
 		       cnp->cn_flags));
 
+#ifdef PDIRUNLOCK
+    cnp->cn_flags &= ~PDIRUNLOCK;
+#endif
+
     error = xfs_lookup_common(ap->a_dvp, cnp, ap->a_vpp);
 
     if (error == ENOENT
@@ -267,6 +268,21 @@ xfs_lookup(struct vop_lookup_args * ap)
     if (cnp->cn_nameiop != LOOKUP && cnp->cn_flags & ISLASTCN)
 	cnp->cn_flags |= SAVENAME;
 
+    if (error == 0 || error == EJUSTRETURN) {
+	if (ap->a_dvp == *(ap->a_vpp)) {
+	    /* if we looked up ourself, do nothing */
+	} else if (!(cnp->cn_flags & ISLASTCN) || !lockparent) {
+	    /* if we isn't last component and is isn't requested,
+	     * return parent unlocked */
+	    xfs_vfs_unlock (ap->a_dvp, xfs_cnp_to_proc(cnp));
+#ifdef PDIRUNLOCK
+	    cnp->cn_flags |= PDIRUNLOCK;
+#endif
+	}
+    } else {
+	/* in case of a error do nothing  */
+    } 
+    
     XFSDEB(XDEBVNOPS, ("xfs_lookup: error = %d\n", error));
 
     return error;
@@ -283,34 +299,52 @@ xfs_cachedlookup(struct vop_cachedlookup_args * ap)
 	struct componentname *a_cnp;
 }; */
 {
-    return xfs_lookup(ap);
+    return xfs_lookup((struct vop_lookup_args *)ap);
 }
 #endif /* HAVE_VOP_CACHEDLOOKUP */
 
+/*
+ * whatever clean-ups are needed for a componentname.
+ */
+
+static void
+cleanup_cnp (struct componentname *cnp, int error)
+{
+    if (error != 0 || (cnp->cn_flags & SAVESTART) == 0) {
+#ifdef HAVE_KERNEL_ZFREEI
+	zfreei(namei_zone, cnp->cn_pnbuf);
+	cnp->cn_flags &= ~HASBUF;
+#elif defined(FREE_ZONE)
+	FREE_ZONE(cnp->cn_pnbuf, cnp->cn_pnlen, M_NAMEI);
+#elif defined(PNBUF_PUT)
+	PNBUF_PUT(cnp->cn_pnbuf);
+#else
+	FREE (cnp->cn_pnbuf, M_NAMEI);
+#endif
+    }
+}
+
 #ifdef HAVE_VOP_CREATE
 static int
-xfs_create(struct vop_create_args * ap)
+xfs_create(struct vop_create_args *ap)
 {
     struct vnode *dvp  = ap->a_dvp;
     struct componentname *cnp = ap->a_cnp;
     const char *name   = cnp->cn_nameptr;
     struct ucred *cred = cnp->cn_cred;
+    struct proc *p     = xfs_cnp_to_proc(cnp);
     int error;
 
-    error = xfs_create_common(dvp,
-			      name,
-			      ap->a_vap,
-			      cred);
+    error = xfs_create_common(dvp, name, ap->a_vap, cred, p);
 
-    if (error == 0)
-	error = xfs_lookup_name(dvp, name, xfs_cnp_to_proc(cnp),
-				cred, ap->a_vpp);
+    if (error == 0) {
+	error = xfs_lookup_common(dvp, cnp, ap->a_vpp);
+    }
 
-    if (error != 0 || (ap->a_cnp->cn_flags & SAVESTART) == 0)
-#ifdef HAVE_KERNEL_ZFREEI
-	zfreei(namei_zone, ap->a_cnp->cn_pnbuf);
-#else
-	free (ap->a_cnp->cn_pnbuf, M_NAMEI);
+    cleanup_cnp (cnp, error);
+
+#if defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
+    vput (dvp);
 #endif
 
     XFSDEB(XDEBVNOPS, ("xfs_create: error = %d\n", error));
@@ -328,15 +362,18 @@ xfs_remove(struct vop_remove_args * ap)
 {
     struct componentname *cnp = ap->a_cnp;
     int error = xfs_remove_common(ap->a_dvp, ap->a_vp, cnp->cn_nameptr, 
-				  cnp->cn_cred);
+				  cnp->cn_cred, xfs_cnp_to_proc(cnp));
 
-    if (error != 0 || (cnp->cn_flags & SAVESTART) == 0)
-#ifdef HAVE_KERNEL_ZFREEI
-	zfreei(namei_zone, cnp->cn_pnbuf);
-#else
-	free (cnp->cn_pnbuf, M_NAMEI);
+#ifdef __APPLE__
+    if (error == 0) {
+	if (UBCINFOEXISTS(ap->a_vp)) {
+	    ubc_setsize(ap->a_vp, 0);
+	    ubc_release(ap->a_vp);
+	    ubc_uncache(ap->a_vp);
+	}
+    }
 #endif
-
+    cleanup_cnp (cnp, error);
     return error;
 }
 #endif /* HAVE_VOP_REMOVE */
@@ -364,7 +401,8 @@ xfs_rename(struct vop_rename_args * ap)
 				  tdvp,
 				  tvp,
 				  ap->a_tcnp->cn_nameptr,
-				  ap->a_tcnp->cn_cred);
+				  ap->a_tcnp->cn_cred,
+				  xfs_cnp_to_proc (ap->a_fcnp));
     if(tdvp == tvp)
 	vrele(tdvp);
     else
@@ -390,30 +428,21 @@ xfs_mkdir(struct vop_mkdir_args * ap)
     struct componentname *cnp = ap->a_cnp;
     const char *name   = cnp->cn_nameptr;
     struct ucred *cred = cnp->cn_cred;
+    struct proc *p     = xfs_cnp_to_proc(cnp);
     int error;
 
-    error = xfs_mkdir_common(dvp,
-			     name,
-			     ap->a_vap,
-			     cred);
+    error = xfs_mkdir_common(dvp, name, ap->a_vap, cred, p);
 
     if (error == 0)
-	error = xfs_lookup_name(dvp, name, xfs_cnp_to_proc(cnp),
-				cred, ap->a_vpp);
+	error = xfs_lookup_common(dvp, cnp, ap->a_vpp);
 
-    if (error != 0 || (ap->a_cnp->cn_flags & SAVESTART) == 0) {
-#ifdef HAVE_KERNEL_ZFREEI
-	zfreei(namei_zone, ap->a_cnp->cn_pnbuf);
-#else
-	free (ap->a_cnp->cn_pnbuf, M_NAMEI);
-#endif
-    }
+    cleanup_cnp (cnp, error);
 
-#if defined(__OpenBSD__) 
-    vput(ap->a_dvp);
+#if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__)
+    vput(dvp);
 #endif
 
-    XFSDEB(XDEBVNOPS, ("xfs_create: error = %d\n", error));
+    XFSDEB(XDEBVNOPS, ("xfs_mkdir: error = %d\n", error));
 
     return error;
 }
@@ -428,20 +457,25 @@ xfs_rmdir(struct vop_rmdir_args * ap)
 {
     struct componentname *cnp = ap->a_cnp;
     int error = xfs_rmdir_common(ap->a_dvp, ap->a_vp, 
-				 cnp->cn_nameptr, cnp->cn_cred);
-
-    if (error != 0 || (cnp->cn_flags & SAVESTART) == 0)
-#ifdef HAVE_KERNEL_ZFREEI
-	zfreei(namei_zone, cnp->cn_pnbuf);
-#else
-	free (cnp->cn_pnbuf, M_NAMEI);
-#endif
+				 cnp->cn_nameptr,
+				 cnp->cn_cred,
+				 xfs_cnp_to_proc(cnp));
 
+    cleanup_cnp (cnp, error);
     return error;
 }
 #endif /* HAVE_VOP_RMDIR */
 
 #ifdef HAVE_VOP_READDIR
+
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
+typedef u_long xfs_cookie_t;
+#elif defined(__NetBSD__)
+typedef off_t xfs_cookie_t;
+#else
+#error dunno want kind of cookies you have
+#endif
+
 static int
 xfs_readdir(struct vop_readdir_args * ap)
      /* struct vnode *vp,
@@ -451,54 +485,39 @@ xfs_readdir(struct vop_readdir_args * ap)
     int error;
     off_t off;
 
-    off =  ap->a_uio->uio_offset;
+    off = ap->a_uio->uio_offset;
 
     error = xfs_readdir_common(ap->a_vp, ap->a_uio, ap->a_cred,
-			      ap->a_eofflag);
+			       xfs_uio_to_proc (ap->a_uio),
+			       ap->a_eofflag);
 
     if (!error && ap->a_ncookies != NULL) {
 	struct uio *uio = ap->a_uio;
-	struct dirent* dp;
+	const struct dirent *dp, *dp_start, *dp_end;
 	int ncookies;
-#if defined(__FreeBSD__) || defined (__OpenBSD__)
-	u_long *cookies;
-	struct dirent* dpStart;
-	struct dirent* dpEnd;
-	u_long *cookiep;
-#else
-	off_t *cookies;
-	cookies = ap->a_cookies;
-	ncookies = ap->a_ncookies;
-#endif	
+	xfs_cookie_t *cookies, *cookiep;
+
 	if (uio->uio_segflg != UIO_SYSSPACE || uio->uio_iovcnt != 1)
 	    panic("xfs_readdir: mail arla-drinkers and tell them to bake burned cookies");
-	dp = (struct dirent *)
-	    (uio->uio_iov->iov_base - (uio->uio_offset - off));
-#if defined(__FreeBSD__) || defined (__OpenBSD__)
-	dpEnd = (struct dirent *) uio->uio_iov->iov_base;
-	for (dpStart = dp, ncookies = 0;
-	     dp < dpEnd;
-	     dp = (struct dirent *)((caddr_t) dp + dp->d_reclen))
+	dp = (const struct dirent *)
+	    ((const char *)uio->uio_iov->iov_base - (uio->uio_offset - off));
+
+	dp_end = (const struct dirent *) uio->uio_iov->iov_base;
+	for (dp_start = dp, ncookies = 0;
+	     dp < dp_end;
+	     dp = (const struct dirent *)((const char *) dp + dp->d_reclen))
 	    ncookies++;
-	MALLOC(cookies, u_long *, ncookies * sizeof(u_long),
-		       M_TEMP, M_WAITOK);
-	for (dp = dpStart, cookiep = cookies;
-	     dp < dpEnd;
-	     dp = (struct dirent *)((caddr_t) dp + dp->d_reclen)) {
+
+	MALLOC(cookies, xfs_cookie_t *, ncookies * sizeof(xfs_cookie_t),
+	       M_TEMP, M_WAITOK);
+	for (dp = dp_start, cookiep = cookies;
+	     dp < dp_end;
+	     dp = (const struct dirent *)((const char *) dp + dp->d_reclen)) {
 	    off += dp->d_reclen;
-	    *cookiep++ = (u_int) off;
+	    *cookiep++ = off;
 	}
 	*ap->a_cookies = cookies;
 	*ap->a_ncookies = ncookies;
-#else /* __NetBSD__ */
-	while (ncookies-- && off < uio->uio_offset) {
-	    if (dp->d_reclen == 0)
-		break;
-	    off += dp->d_reclen;
-	    *(cookies++) = off;
-	    dp = (struct dirent *)((caddr_t)dp + dp->d_reclen);
-	}
-#endif
     }
     return error;
 }
@@ -521,40 +540,47 @@ xfs_link(struct vop_link_args * ap)
 
 #if defined (__OpenBSD__) || defined(__NetBSD__)
     dvp = ap->a_dvp;
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__APPLE__)
     dvp = ap->a_tdvp;
+#else
+#error what kind of BSD is this?
 #endif
 
     if (vp->v_type == VDIR) {
+#ifdef HAVE_VOP_ABORTOP
 	    VOP_ABORTOP(dvp, cnp);
-	    return EPERM;
+#endif
+	    error = EPERM;
+	    goto out;
     }
     if (dvp->v_mount != vp->v_mount) {
+#ifdef HAVE_VOP_ABORTOP
 	    VOP_ABORTOP(dvp, cnp);
-	    return EXDEV;
+#endif
+	    error = EXDEV;
+	    goto out;
     }
     if (dvp != vp && (error = xfs_vfs_writelock(vp, p))) {
+#ifdef HAVE_VOP_ABORTOP
 	    VOP_ABORTOP(dvp, cnp);
-	    return error;
+#endif
+	    goto out;
     }
 
     error = xfs_link_common(
 			   dvp,
 			   vp,
 			   cnp->cn_nameptr,
-			   cnp->cn_cred);
+			   cnp->cn_cred,
+			   xfs_cnp_to_proc (cnp));
 
-    if (error != 0 || (cnp->cn_flags & SAVESTART) == 0)
-#ifdef HAVE_KERNEL_ZFREEI
-	zfreei(namei_zone, cnp->cn_pnbuf);
-#else
-	free (cnp->cn_pnbuf, M_NAMEI);
-#endif
+    cleanup_cnp (cnp, error);
 
     if (dvp != vp)
 	xfs_vfs_unlock(vp, p);
 
-#if defined(__OpenBSD__) || defined(__NetBSD__) || (defined(__FreeBSD__) && !defined(__FreeBSD_version))
+out:
+#if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__)
     vput(dvp);
 #endif
 
@@ -576,19 +602,11 @@ xfs_symlink(struct vop_symlink_args * ap)
     struct componentname *cnp = ap->a_cnp;
     int error = xfs_symlink_common(ap->a_dvp,
 				   ap->a_vpp,
-				   cnp->cn_nameptr,
-				   xfs_cnp_to_proc(cnp),
-				   cnp->cn_cred,
+				   cnp,
 				   ap->a_vap,
 				   ap->a_target);
 
-    if (error != 0 || (cnp->cn_flags & SAVESTART) == 0) {
-#ifdef HAVE_KERNEL_ZFREEI
-	zfreei(namei_zone, cnp->cn_pnbuf);
-#else
-	free (cnp->cn_pnbuf, M_NAMEI);
-#endif
-    }
+    cleanup_cnp (cnp, error);
     return error;
 }
 #endif /* HAVE_VOP_SYMLINK */
@@ -611,15 +629,7 @@ xfs_inactive(struct vop_inactive_args * ap)
      /*struct vnode *vp,
 	     struct ucred *cred)*/
 {
-#if 0
-    /*
-     * This break freebsd 2.2.x
-     */
-    
     return xfs_inactive_common(ap->a_vp, xfs_curproc());
-#else
-    return 0;
-#endif
 }
 #endif /* HAVE_VOP_INACTICE */
 
@@ -631,26 +641,121 @@ xfs_reclaim(struct vop_reclaim_args * ap)
 	struct vnode *a_vp;
 };*/
 {
-    return xfs_reclaim_common(ap->a_vp);
+    struct vnode *vp = ap->a_vp;
+    int ret;
+
+    ret = xfs_reclaim_common(vp);
+    vp->v_data = NULL;
+    return ret;
 }
 #endif /* HAVE_VOP_RECLAIM */
 
 /*
- * The lock, unlock, islocked vnode operations.
- *
- * This should be done with the generic locking function for the
- * appropriate dialect of BSD (vop_nolock, genfs_nolock, ...).  But,
- * most of these functions are commented out and don't work enough to
- * allow xfs_islocked to figure if the vnode is locked or not, which
- * other parts of the kernel depend on.
- *
- * We try do locking the folling way:
- *  If we have LK_INTERLOCK & co:
- *    Do no locking with (we should use lockmgr)
- *  else
- *    When VXLOCK is set loop
+ * Do lock, unlock, and islocked with lockmgr if we have it.
  */
 
+#if defined(HAVE_KERNEL_LOCKMGR) || defined(HAVE_KERNEL_DEBUGLOCKMGR)
+
+#ifdef HAVE_VOP_LOCK
+static int
+xfs_lock(struct vop_lock_args * ap)
+{               
+    struct vnode *vp    = ap->a_vp;
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    xfs_vnode_lock *l   = &xn->lock;
+    int flags           = ap->a_flags;
+    int ret;
+
+    XFSDEB(XDEBVNOPS, ("xfs_lock: %lx, flags 0x%x\n",
+		       (unsigned long)vp, flags));
+
+    if (l == NULL)
+      panic("xfs_lock: lock NULL");
+
+    XFSDEB(XDEBVNOPS, ("xfs_lock before: lk flags: %d share: %d "
+		       "wait: %d excl: %d holder: %d\n",
+		       l->lk_flags, l->lk_sharecount, l->lk_waitcount,
+		       l->lk_exclusivecount, l->lk_lockholder));
+
+#ifndef	DEBUG_LOCKS
+#ifdef HAVE_FOUR_ARGUMENT_LOCKMGR
+    ret = lockmgr(l, flags, &vp->v_interlock, ap->a_p);
+#else
+    ret = lockmgr(l, flags, &vp->v_interlock);
+#endif
+#else
+    ret = debuglockmgr(l, flags, &vp->v_interlock, ap->a_p,
+			"xfs_lock", ap->a_vp->filename, ap->a_vp->line);
+#endif
+    XFSDEB(XDEBVNOPS, ("xfs_lock: lk flags: %d share: %d "
+		       "wait: %d excl: %d holder: %d\n",
+		       l->lk_flags, l->lk_sharecount, l->lk_waitcount,
+		       l->lk_exclusivecount, l->lk_lockholder));
+    return ret;
+}
+#endif /* HAVE_VOP_LOCK */
+
+#ifdef HAVE_VOP_UNLOCK
+static int
+xfs_unlock(struct vop_unlock_args * ap)
+{
+    struct vnode *vp    = ap->a_vp;
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    xfs_vnode_lock *l   = &xn->lock;
+    int flags           = ap->a_flags;
+    int ret;
+
+    if (l == NULL)
+      panic("xfs_unlock: lock NULL");
+
+    XFSDEB(XDEBVNOPS,
+	   ("xfs_unlock: %lx, flags 0x%x, l %lx, ap %lx\n",
+	    (unsigned long)vp, flags,
+	    (unsigned long)l,
+	    (unsigned long)ap));
+
+    XFSDEB(XDEBVNOPS, ("xfs_unlock: lk flags: %d share: %d "
+		       "wait: %d excl: %d holder: %d\n",
+		       l->lk_flags, l->lk_sharecount, l->lk_waitcount,
+		       l->lk_exclusivecount, l->lk_lockholder));
+#ifndef	DEBUG_LOCKS
+#ifdef HAVE_FOUR_ARGUMENT_LOCKMGR
+    ret = lockmgr (l, flags | LK_RELEASE, &vp->v_interlock, ap->a_p);
+#else
+    ret = lockmgr (l, flags | LK_RELEASE, &vp->v_interlock);
+#endif
+#else
+    ret = debuglockmgr (l, flags | LK_RELEASE, &vp->v_interlock, ap->a_p,
+			"xfs_lock", ap->a_vp->filename, ap->a_vp->line);
+#endif
+    XFSDEB(XDEBVNOPS, ("xfs_unlock: return %d\n", ret));
+    return ret;
+}
+#endif /* HAVE_VOP_UNLOCK */
+
+#ifdef HAVE_VOP_ISLOCKED
+static int
+xfs_islocked (struct vop_islocked_args *ap)
+{
+    struct vnode *vp    = ap->a_vp;
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    xfs_vnode_lock *l   = &xn->lock;
+
+    XFSDEB(XDEBVNOPS, ("xfs_islocked: %lx\n",
+		       (unsigned long)vp));
+
+#if defined(HAVE_TWO_ARGUMENT_LOCKSTATUS)
+    return lockstatus (l, ap->a_p);
+#elif defined(HAVE_ONE_ARGUMENT_LOCKSTATUS)
+    return lockstatus (l);
+#else
+#error what lockstatus?
+#endif
+}
+#endif /* HAVE_VOP_ISLOCKED */
+
+#else /* !HAVE_KERNEL_LOCKMGR && !HAVE_KERNEL_DEBUGLOCKMGR */
+
 #ifdef HAVE_VOP_LOCK
 static int
 xfs_lock(struct vop_lock_args * ap)
@@ -658,26 +763,15 @@ xfs_lock(struct vop_lock_args * ap)
     struct vnode *vp    = ap->a_vp;
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
 
-    XFSDEB(XDEBVNOPS, ("xfs_lock: %p, %d\n", vp, xn->vnlocks));
+    XFSDEB(XDEBVNOPS, ("xfs_lock: %lx, %d\n",
+		       (unsigned long)vp, xn->vnlocks));
 
-#if defined(HAVE_LK_INTERLOCK) && !defined(HAVE_ONE_ARGUMENT_VOP_LOCK)
-    {
-	int flags = ap->a_flags;
-
-	if (flags & LK_INTERLOCK)
-	    simple_unlock(&vp->v_interlock);
-
-	if (!(flags & LK_TYPE_MASK))
-	    return 0;
-    }
-#else
     while (vp->v_flag & VXLOCK) {
 	vp->v_flag |= VXWANT;
 	(void) tsleep((caddr_t)vp, PINOD, "xfs_vnlock", 0);
     }
     if (vp->v_tag == VT_NON)
 	return (ENOENT);
-#endif
     ++xn->vnlocks;
     return 0;
 }
@@ -689,23 +783,16 @@ xfs_unlock(struct vop_unlock_args * ap)
 {
     struct vnode *vp    = ap->a_vp;
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
-    XFSDEB(XDEBVNOPS, ("xfs_unlock: %p, %d\n", vp, xn->vnlocks));
+    XFSDEB(XDEBVNOPS, ("xfs_unlock: %lx, %d\n",
+		       (unsigned long)vp, xn->vnlocks));
 
-#if defined(HAVE_LK_INTERLOCK) && !defined(HAVE_ONE_ARGUMENT_VOP_LOCK)
-    {
-	int flags = ap->a_flags;
-
-	if (flags & LK_INTERLOCK)
-	    simple_unlock(&vp->v_interlock);
-	if (!(flags & LK_TYPE_MASK))
-	    return 0;
-    }
-#endif
     --xn->vnlocks;
     if (xn->vnlocks < 0) {
 	printf ("PANIC: xfs_unlock: unlocking unlocked\n");
 	xn->vnlocks = 0;
     }
+    XFSDEB(XDEBVNOPS, ("xfs_unlock: return\n"));
+
     return 0;
 }
 #endif /* HAVE_VOP_UNLOCK */
@@ -717,11 +804,13 @@ xfs_islocked (struct vop_islocked_args *ap)
     struct vnode *vp    = ap->a_vp;
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
 
-    XFSDEB(XDEBVNOPS, ("xfs_islocked: %p, %d\n", vp, xn->vnlocks));
+    XFSDEB(XDEBVNOPS, ("xfs_islocked: %lx, %d\n",
+		       (unsigned long)vp, xn->vnlocks));
 
     return xn->vnlocks;
 }
 #endif /* HAVE_VOP_ISLOCKED */
+#endif /* !HAVE_KERNEL_LOCKMGR */
 
 #ifdef HAVE_VOP_ABORTOP
 static int
@@ -734,6 +823,11 @@ xfs_abortop (struct vop_abortop_args *ap)
     if ((cnp->cn_flags & (HASBUF | SAVESTART)) == HASBUF)
 #ifdef HAVE_KERNEL_ZFREEI
 	zfreei(namei_zone, cnp->cn_pnbuf);
+	ap->a_cnp->cn_flags &= ~HASBUF;
+#elif defined(FREE_ZONE)
+	FREE_ZONE(cnp->cn_pnbuf, cnp->cn_pnlen, M_NAMEI);
+#elif defined(PNBUF_PUT)
+	PNBUF_PUT(cnp->cn_pnbuf);
 #else
 	FREE(cnp->cn_pnbuf, M_NAMEI);
 #endif
@@ -775,13 +869,23 @@ xfs_bmap(struct vop_bmap_args *ap)
 #ifdef HAVE_VOP_GETPAGES
 static int
 xfs_getpages (struct vop_getpages_args *ap)
-     /*
+     /* Old BSD
 	IN struct vnode *vp;
 	IN vm_page_t *m;
 	IN int count;
 	IN int reqpage;
 	IN vm_ooffset_t offset;
-	*/
+     */
+    /* NetBSD UBC
+	IN struct vnode *vp;
+	IN voff_t offset;
+	IN vm_page_t *m;
+	IN int *count;
+	IN int centeridx;
+	IN vm_prot_t access_type;
+	IN int advice;
+	IN int flags;
+    */
 {
     int error;
 
@@ -791,7 +895,13 @@ xfs_getpages (struct vop_getpages_args *ap)
     error = vnode_pager_generic_getpages (ap->a_vp, ap->a_m, 
 					  ap->a_count, ap->a_reqpage);
 #else
-    error = EOPNOTSUPP;
+    error = xfs_data_valid (ap->a_vp, VNODE_TO_XNODE(ap->a_vp)->cred,
+			    xfs_curproc(), XFS_DATA_R);
+    if (error == 0)
+	error = VOP_GETPAGES(DATA_FROM_VNODE(ap->a_vp), 
+			     ap->a_offset, ap->a_m,
+			     ap->a_count, ap->a_centeridx, ap->a_access_type,
+			     ap->a_advice, ap->a_flags);
 #endif
     XFSDEB(XDEBVNOPS, ("xfs_getpages = %d\n", error));
     return error;
@@ -801,22 +911,36 @@ xfs_getpages (struct vop_getpages_args *ap)
 #ifdef HAVE_VOP_PUTPAGES
 static int
 xfs_putpages (struct vop_putpages_args *ap)
-     /*
+     /* Old BSD
         IN struct vnode *vp;
         IN vm_page_t *m;
         IN int count;
         IN int sync;
         IN int *rtvals;
         IN vm_ooffset_t offset;
-	*/
+     */
+    /* NetBSD UBC
+       IN struct vnode *vp;
+       IN vm_page_t *m;
+       IN int count;
+       IN int flags;
+       IN int *rtvals;
+    */
 {
+    struct vnode *vp    = ap->a_vp;
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    struct vnode *t     = DATA_FROM_XNODE(xn);
+    int error;
+
     XFSDEB(XDEBVNOPS, ("xfs_putpages\n"));
 
-#if HAVE_KERNEL_VNODE_PAGER_GENERIC_PUTPAGES
-    return vnode_pager_generic_putpages (ap->a_vp, ap->a_m, ap->a_count, 
-					 ap->a_sync, ap->a_rtvals);
-#else
-    return EOPNOTSUPP;
+    xn->flags |= XFS_DATA_DIRTY;
+
+#ifdef HAVE_STRUCT_VOP_PUTPAGES_ARGS_A_SYNC /* FreeBSD-style */
+    return VOP_PUTPAGES(t, ap->a_m, ap->a_count, ap->a_sync, ap->a_rtvals,
+			ap->a_offset);
+#else /* NetBSD-style */
+    return VOP_PUTPAGES(t, ap->a_m, ap->a_count, ap->a_flags, ap->a_rtvals);
 #endif
 }
 #endif /* HAVE_VOP_PUTPAGES */
@@ -916,19 +1040,150 @@ xfs_revoke(void *v)
 }
 #endif /* HAVE_VOP_REVOKE */
 
+#ifdef HAVE_VOP_PAGEIN
+static int
+xfs_pagein(struct vop_pagein_args *ap)
+{
+#ifdef __APPLE__
+    struct uio uio;
+    struct iovec iov;
+    int ret;
+
+    kernel_upl_map(kernel_map, ap->a_pl, &iov.iov_base);
+    iov.iov_base+=ap->a_pl_offset;
+    iov.iov_len=ap->a_size;
+
+    uio.uio_iov=&iov;
+    uio.uio_iovcnt=1;
+    uio.uio_offset=ap->a_f_offset;
+    uio.uio_resid=ap->a_size;
+    uio.uio_segflg=UIO_SYSSPACE; /* XXX what is it? */
+    uio.uio_rw=UIO_READ;
+    uio.uio_procp=xfs_curproc();
+
+    ret = VOP_READ(ap->a_vp, &uio, 0, ap->a_cred);
+
+    kernel_upl_unmap(kernel_map, ap->a_pl);
+
+    if (ret) {
+	kernel_upl_abort_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
+			       UPL_ABORT_ERROR | UPL_ABORT_FREE_ON_EMPTY);
+    } else {
+	kernel_upl_commit_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
+				UPL_COMMIT_CLEAR_DIRTY | UPL_COMMIT_FREE_ON_EMPTY,
+				UPL_GET_INTERNAL_PAGE_LIST(ap->a_pl));
+    }
+
+    return ret;
+#else
+    return (VOP_READ(ap->a_vp, ap->a_uio, ap->a_ioflag, ap->a_cred));
+#endif
+}
+  
+#endif
+
+#ifdef HAVE_VOP_PAGEOUT
+static int
+xfs_pageout(struct vop_pageout_args *ap)
+{
+#ifdef __APPLE__
+    struct uio uio;
+    struct iovec iov;
+    int ret;
+
+    kernel_upl_map(kernel_map, ap->a_pl, &iov.iov_base);
+    iov.iov_base+=ap->a_pl_offset;
+    iov.iov_len=ap->a_size;
+
+    uio.uio_iov=&iov;
+    uio.uio_iovcnt=1;
+    uio.uio_offset=ap->a_f_offset;
+    uio.uio_resid=ap->a_size;
+    uio.uio_segflg=UIO_SYSSPACE; /* XXX what is it? */
+    uio.uio_rw=UIO_WRITE;
+    uio.uio_procp=xfs_curproc();
+
+    ret = VOP_WRITE(ap->a_vp, &uio, 0, ap->a_cred);
+
+    kernel_upl_unmap(kernel_map, ap->a_pl);
+
+    if (ret) {
+	kernel_upl_abort_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
+			       UPL_ABORT_FREE_ON_EMPTY);
+    } else {
+	kernel_upl_commit_range(ap->a_pl, ap->a_pl_offset, ap->a_size,
+				UPL_COMMIT_CLEAR_DIRTY | UPL_COMMIT_FREE_ON_EMPTY,
+				UPL_GET_INTERNAL_PAGE_LIST(ap->a_pl));
+    }
+
+    return ret;
+#else
+    return (VOP_WRITE(ap->a_vp, ap->a_uio, ap->a_ioflag, ap->a_cred));
+#endif
+}
+#endif
+
+#ifdef HAVE_VOP_CREATEVOBJECT
+static int
+xfs_createvobject(struct vop_createvobject_args *ap)
+/*
+struct vop_createvobject_args {
+	struct vnode *vp;
+	struct ucred *cred;
+	struct proc *p;
+};
+ */
+{
+    XFSDEB(XDEBVNOPS, ("xfs_createvobject\n"));
+
+    return vop_stdcreatevobject (ap);
+}
+#endif /* HAVE_VOP_CREATEVOBJECT */
+
+#ifdef HAVE_VOP_DESTROYVOBJECT
+static int
+xfs_destroyvobject(struct vop_destroyvobject_args *ap)
+/*
+struct vop_destroyvobject_args {
+	struct vnode *vp;
+};
+ */
+{
+    XFSDEB(XDEBVNOPS, ("xfs_destroyvobject\n"));
+
+    return vop_stddestroyvobject (ap);
+}
+#endif /* HAVE_VOP_DESTROYVOBJECT */
+
+#ifdef HAVE_VOP_GETVOBJECT
+static int
+xfs_getvobject(struct vop_getvobject_args *ap)
+/*
+struct vop_getvobject_args {
+	struct vnode *vp;
+	struct vm_object **objpp;
+};
+ */
+{
+    XFSDEB(XDEBVNOPS, ("xfs_getvobject\n"));
+
+    return vop_stdgetvobject (ap);
+}
+#endif /* HAVE_VOP_GETVOBJECT */
+
 vop_t **xfs_vnodeop_p;
 
 int
-xfs_eopnotsupp (void *v)
+xfs_eopnotsupp (struct vop_generic_args *ap)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_eopnotsupp\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_eopnotsupp %s\n", ap->a_desc->vdesc_name));
     return EOPNOTSUPP;
 }
 
 int
-xfs_returnzero (void *v)
+xfs_returnzero (struct vop_generic_args *ap)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_returnzero\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_returnzero %s\n", ap->a_desc->vdesc_name));
     return 0;
 }
 
@@ -1047,6 +1302,21 @@ static struct vnodeopv_entry_desc xfs_vnodeop_entries[] = {
 #ifdef HAVE_VOP_ADVLOCK
     {&vop_advlock_desc, (vop_t *) xfs_advlock },
 #endif
+#endif
+#ifdef HAVE_VOP_PAGEIN
+    {&vop_pagein_desc, (vop_t *) xfs_pagein },
+#endif
+#ifdef HAVE_VOP_PAGEOUT
+    {&vop_pageout_desc, (vop_t *) xfs_pageout },
+#endif
+#ifdef HAVE_VOP_CREATEVOBJECT
+    {&vop_createvobject_desc, (vop_t *) xfs_createvobject },
+#endif
+#ifdef HAVE_VOP_DESTROYVOBJECT
+    {&vop_destroyvobject_desc, (vop_t *) xfs_destroyvobject },
+#endif
+#ifdef HAVE_VOP_GETVOBJECT
+    {&vop_getvobject_desc, (vop_t *) xfs_getvobject },
 #endif
     {(struct vnodeop_desc *) NULL, (int (*) (void *)) NULL}
 };
