@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -52,7 +47,7 @@
 
 #include <sys/buf.h>
 
-RCSID("$Id: xfs_vnodeops-osf.c,v 1.13 1999/06/28 07:49:05 joda Exp $");
+RCSID("$Id: xfs_vnodeops-osf.c,v 1.23 2000/10/02 23:33:41 lha Exp $");
 
 /*
  * vnode functions
@@ -61,12 +56,7 @@ RCSID("$Id: xfs_vnodeops-osf.c,v 1.13 1999/06/28 07:49:05 joda Exp $");
 static int
 xfs_open(struct vnode **vpp, int mode, struct ucred *cred)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_open\n"));
-
-    if (mode & FWRITE)
-	return xfs_open_valid(*vpp, cred, XFS_OPEN_NW);
-    else
-	return xfs_open_valid(*vpp, cred, XFS_OPEN_NR);
+    return xfs_open_common (*vpp, mode, cred, xfs_curproc());
 }
 
 static int
@@ -107,19 +97,19 @@ xfs_ioctl(struct vnode *vp,
 static int
 xfs_getattr(struct vnode *vp, struct vattr *vap, struct ucred *cred)
 {
-    return xfs_getattr_common(vp, vap, cred);
+    return xfs_getattr_common(vp, vap, cred, xfs_curproc ());
 }
 
 static int
 xfs_setattr(struct vnode *vp, struct vattr *vap, struct ucred *cred)
 {
-    return xfs_setattr_common (vp, vap, cred);
+    return xfs_setattr_common (vp, vap, cred, xfs_curproc ());
 }
 
 static int
 xfs_access(struct vnode *vp, int mode, struct ucred *cred)
 {
-    return xfs_access_common(vp, mode, cred);
+    return xfs_access_common(vp, mode, cred, xfs_curproc());
 }
 
 static int
@@ -142,10 +132,10 @@ xfs_create(struct nameidata *ndp, struct vattr *vap)
     error = xfs_create_common(dvp,
 			      name,
 			      vap,
-			      cred);
+			      cred,
+			      xfs_curproc());
     if(error == 0)
-	error = xfs_lookup_name (dvp, name, NULL, cred,
-				 &ndp->ni_vp);
+	error = xfs_lookup_common (dvp, ndp, &ndp->ni_vp);
 
     XFSDEB(XDEBVNOPS, ("xfs_create: error = %d\n", error));
     
@@ -158,19 +148,22 @@ xfs_remove(struct nameidata *ndp)
     return xfs_remove_common(ndp->ni_dvp, 
 			     ndp->ni_vp, 
 			     ndp->ni_dent.d_name, 
-			     ndp->ni_cred);
+			     ndp->ni_cred,
+			     xfs_curproc());
 }
 
 static int
 xfs_rename(struct nameidata *from, struct nameidata *to)
 {
     int error =  xfs_rename_common(from->ni_dvp, 
-			     from->ni_vp, 
-			     from->ni_dent.d_name, 
-			     to->ni_dvp, 
-			     to->ni_vp, 
-			     to->ni_dent.d_name, 
-			     from->ni_cred);
+				   from->ni_vp, 
+				   from->ni_dent.d_name, 
+				   to->ni_dvp, 
+				   to->ni_vp, 
+				   to->ni_dent.d_name, 
+				   from->ni_cred,
+				   xfs_curproc());
+    
     vrele(to->ni_dvp);
     if (to->ni_vp)
 	vrele(to->ni_vp);
@@ -192,11 +185,11 @@ xfs_mkdir(struct nameidata *ndp,
     error = xfs_mkdir_common(dvp,
 			     name,
 			     vap,
-			     cred);
+			     cred,
+			     xfs_curproc());
 
     if(error == 0)
-	error = xfs_lookup_name (dvp, name, NULL, cred,
-				 &ndp->ni_vp);
+	error = xfs_lookup_common (dvp, ndp, &ndp->ni_vp);
 
     XFSDEB(XDEBVNOPS, ("xfs_mkdir: error = %d\n", error));
     
@@ -209,7 +202,8 @@ xfs_rmdir(struct nameidata *ndp)
     return xfs_rmdir_common(ndp->ni_dvp, 
 			    ndp->ni_vp, 
 			    ndp->ni_dent.d_name, 
-			    ndp->ni_cred);
+			    ndp->ni_cred,
+			    xfs_curproc());
 }
 
 static int
@@ -218,21 +212,21 @@ xfs_readdir(struct vnode *vp,
 	    struct ucred *cred, 
 	    int *eofflagp)
 {
-    return xfs_readdir_common(vp, uio, cred, eofflagp);
+    return xfs_readdir_common(vp, uio, cred, xfs_curproc(), eofflagp);
 }
 
 static int
 xfs_link(struct vnode* vp, struct nameidata *ndp)
 {
-    return xfs_link_common(ndp->ni_dvp, vp, ndp->ni_dent.d_name, ndp->ni_cred);
+    return xfs_link_common(ndp->ni_dvp, vp, ndp->ni_dent.d_name,
+			   ndp->ni_cred, xfs_curproc());
 }
 
 static int
 xfs_symlink(struct nameidata *ndp, struct vattr *vap, char *target)
 {
     return xfs_symlink_common(ndp->ni_dvp, &ndp->ni_vp,
-			      ndp->ni_dent.d_name, NULL,
-			      ndp->ni_cred, vap, target);
+			      ndp, vap, target);
 }
 
 static int
@@ -244,7 +238,7 @@ xfs_readlink(struct vnode *vp, struct uio *uiop, struct ucred *cred)
 static int
 xfs_inactive(struct vnode *vp)
 {
-    return xfs_inactive_common(vp, NULL);
+    return xfs_inactive_common(vp, xfs_curproc());
 }
 
 static int
@@ -256,14 +250,18 @@ xfs_reclaim(struct vnode *vp)
 static int
 xfs_lock(struct vnode *vp, int flag, struct ucred *cred)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_lock(%p, %d, %p)\n", vp, flag, cred));
+    XFSDEB(XDEBVNOPS, ("xfs_lock(%lx, %d, %lx)\n",
+		       (unsigned long)vp, flag,
+		       (unsigned long)cred));
     return EOPNOTSUPP;
 }
 
 static int
 xfs_unlock(struct vnode *vp, int flag, struct ucred *cred)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_unlock(%p, %d, %p)\n", vp, flag, cred));
+    XFSDEB(XDEBVNOPS, ("xfs_unlock(%lx, %d, %lx)\n",
+		       (unsigned long)vp, flag,
+		       (unsigned long)cred));
     return EOPNOTSUPP;
 }
 
@@ -290,9 +288,10 @@ xfs_mmap(struct vnode *vp,
 				     struct vm_ubc_object *, 
 				     vm_offset_t);
     
-    XFSDEB(XDEBVNOPS, ("xfs_mmap (%p, %lx, %p, %p, %lx, %x, %x, %d, %p)\n",
-		       vp, offset, map, addrp,
-		       len, prot, maxprot, flags, cred));
+    XFSDEB(XDEBVNOPS, ("xfs_mmap (%lx, %lx, %lx, %lx, %lx, %x, %x, %d, %lx)\n",
+		       (unsigned long)vp, offset, map, addrp,
+		       len, prot, maxprot, flags,
+		       (unsigned long)cred));
     
     args.a_offset = offset;
     args.a_vaddr = addrp;
@@ -326,24 +325,31 @@ int
 xfs_getproplist (struct vnode *vp, char **names, struct uio *uio, 
 		 int *min_buf_size, int mask, struct ucred *cred)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_getproplist(%p, %p, %p, %p, %d, %p)\n",
-		       vp, names, uio, min_buf_size, mask, cred));
+    XFSDEB(XDEBVNOPS, ("xfs_getproplist(%lx, %lx, %lx, %lx, %d, %lx)\n",
+		       (unsigned long)vp,
+		       (unsigned long)names,
+		       (unsigned long)uio, min_buf_size, mask,
+		       (unsigned long)cred));
     return EOPNOTSUPP;
 }
 
 int
 xfs_setproplist (struct vnode *vp, struct uio *uio, struct ucred *cred)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_setproplist(%p, %p, %p)\n",
-		       vp, uio, cred));
+    XFSDEB(XDEBVNOPS, ("xfs_setproplist(%lx, %lx, %lx)\n",
+		       (unsigned long)vp,
+		       (unsigned long)uio,
+		       (unsigned long)cred));
     return EOPNOTSUPP;
 }
 
 int
 xfs_delproplist (struct vnode *vp, char **names, int mask, struct ucred *cred)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_delproplist(%p, %p, %d, %p)\n",
-		       vp, names, mask, cred));
+    XFSDEB(XDEBVNOPS, ("xfs_delproplist(%lx, %lx, %d, %lx)\n",
+		       (unsigned long)vp,
+		       (unsigned long)names, mask,
+		       (unsigned long)cred));
     return EOPNOTSUPP;
 }
 
@@ -402,7 +408,9 @@ struct vnodeops xfs_vnodeops = {
 int
 xfs_refer(vm_ubc_object_t vop)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_refer: vop = %p, vp = %p\n", vop, vop->vu_vp));
+    XFSDEB(XDEBVNOPS, ("xfs_refer: vop = %lx, vp = %lx\n",
+		       (unsigned long)vop,
+		       (unsigned long)vop->vu_vp));
     VREF(vop->vu_vp);
     return 0;
 }
@@ -410,11 +418,14 @@ xfs_refer(vm_ubc_object_t vop)
 int
 xfs_release(vm_ubc_object_t vop)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_release: vop = %p, vp = %p\n", vop, vop->vu_vp));
+    XFSDEB(XDEBVNOPS, ("xfs_release: vop = %lx, vp = %lx\n",
+		       (unsigned long)vop,
+		       (unsigned long)vop->vu_vp));
     vrele(vop->vu_vp);
     return 0;
 }
 
+#ifdef HAVE_SIX_ARGUMENT_UBC_LOOKUP
 int
 xfs_getpage(vm_ubc_object_t vop,
 	    vm_offset_t offset,
@@ -426,6 +437,19 @@ xfs_getpage(vm_ubc_object_t vop,
 	    vm_offset_t addr,
 	    int rwflg,
 	    struct ucred *cred)
+#else
+/* OSF5 */
+int
+xfs_getpage(vm_ubc_object_t vop,
+	    vm_offset_t offset,
+	    vm_size_t len,
+	    vm_prot_t *protp,
+	    vm_page_t *pl,
+	    int plsz,
+	    struct vm_policy *vmp,
+	    int rwflg,
+	    struct ucred *cred)
+#endif
 {
 #if 1
     vm_page_t app, pp;
@@ -433,18 +457,19 @@ xfs_getpage(vm_ubc_object_t vop,
     vm_offset_t off;
     int length;
     
-    
     int error = 0;
     
     struct vnode *vp = vop->vu_vp;
     struct vnode *t = DATA_FROM_VNODE(vp);
     struct vattr attr;
 	    
-    XFSDEB(XDEBVNOPS, ("xfs_getpage: %p, %lx, %ld\n", 
-		       vop, offset, len));
-    XFSDEB(XDEBVNOPS, ("xfs_getpage: %p, %p, %p, %d\n", 
-		       protp, ep, addr, rwflg));
-    error = xfs_data_valid(vp, cred, XFS_DATA_R);
+#if 0
+    XFSDEB(XDEBVNOPS, ("xfs_getpage: %lx, %lx, %ld\n", 
+		       (unsigned long)vop, offset, len));
+    XFSDEB(XDEBVNOPS, ("xfs_getpage: %lx, %lx, %lx, %d\n", 
+		       (unsigned long)protp, ep, addr, rwflg));
+#endif
+    error = xfs_data_valid(vp, cred, xfs_curproc(), XFS_DATA_R);
     if(error)
 	return error;
 
@@ -455,9 +480,11 @@ xfs_getpage(vm_ubc_object_t vop,
     if(offset + len > attr.va_size) {
 	if(offset >= attr.va_size) {
 	    *pl = VM_PAGE_NULL;
+#ifdef HAVE_SIX_ARGUMENT_UBC_LOOKUP
 	    if(ep) {
 		error = KERN_INVALID_ADDRESS;
 	    } else
+#endif
 		error = 0;
 	    return error;
 	}
@@ -472,7 +499,11 @@ xfs_getpage(vm_ubc_object_t vop,
 	    break;
 	}
 	XFSDEB(XDEBVNOPS, ("get page [%lx,%lx)\n", off, length + off));
+#ifdef HAVE_SIX_ARGUMENT_UBC_LOOKUP
 	error = ubc_lookup(vop, off, PAGE_SIZE, length + off, &app, &uflags);
+#else
+	error = ubc_lookup(vop, off, PAGE_SIZE, length, &app, &uflags, vmp);
+#endif
 	if(error) {
 	    XFSDEB(XDEBVNOPS, ("ubc_lookup = %d\n", error));
 	    *pl = VM_PAGE_NULL;
@@ -510,7 +541,7 @@ xfs_getpage(vm_ubc_object_t vop,
 		uio.uio_segflg = UIO_PHYSSPACE;
 		uio.uio_rw = UIO_READ;
 		if(xfsdeb & XDEBVNOPS) {
-		    printf("uio.uio_iov = %p\n", uio.uio_iov);
+		    printf("uio.uio_iov = %lx\n", uio.uio_iov);
 		    printf("uio.uio_offset = %lx\n", uio.uio_offset);
 		    printf("uio.uio_iovcnt = %d\n", uio.uio_iovcnt);
 		    printf("uio.uio_resid = %lx\n", uio.uio_resid);
@@ -542,10 +573,10 @@ xfs_getpage(vm_ubc_object_t vop,
     return error;
     
 #else
-    printf("xfs_getpage: vop = %p, offset = %lx, len = %ld\n", 
-	   vop, offset, len);
-    printf("xfs_getpage: protp = %p, ep = %p, addr = %p, rwflg = %d\n", 
-	   protp, ep, addr, rwflg);
+    printf("xfs_getpage: vop = %lx, offset = %lx, len = %ld\n", 
+	   (unsigned long)vop, offset, len);
+    printf("xfs_getpage: protp = %lx, ep = %lx, addr = %lx, rwflg = %d\n", 
+	   (unsigned long)protp, ep, addr, rwflg);
     *pl = VM_PAGE_NULL;
     return EINVAL;
 #endif
@@ -560,6 +591,7 @@ xfs_putpage(vm_ubc_object_t vop,
 {
 #if 1
     panic("xfs_putpage");
+    return EOPNOTSUPP;
 #else
     printf("xfs_putpage\n");
     return EINVAL;
@@ -569,8 +601,10 @@ xfs_putpage(vm_ubc_object_t vop,
 int
 xfs_write_check(vm_ubc_object_t vpp, vm_page_t pp)
 {
-    XFSDEB(XDEBVNOPS, ("xfs_write_check: vpp = %p, vp = %p, pp = %p\n", 
-		       vpp, vpp->vu_vp, pp));
+    XFSDEB(XDEBVNOPS, ("xfs_write_check: vpp = %lx, vp = %lx, pp = %lx\n", 
+		       (unsigned long)vpp,
+		       (unsigned long)vpp->vu_vp,
+		       (unsigned long)pp));
     return TRUE;
 }
 
