@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -39,89 +34,57 @@
 #define __NO_VERSION__
 
 #include <xfs/xfs_locl.h>
-#include <xfs/xfs_cache.h>
 #include <xfs/xfs_dev.h>
 #include <xfs/xfs_syscalls.h>
 #include <xfs/nxfs.h>
 
-RCSID("$Id: xfs_vfsops.c,v 1.41 1999/07/20 08:27:04 assar Exp $");
+RCSID("$Id: xfs_vfsops.c,v 1.71 2000/11/16 06:09:41 lha Exp $");
 
 struct xfs xfs[NXFS];
 
 static void xfs_read_inode(struct inode *inode);
 static void xfs_delete_inode(struct inode *inode);
 static void xfs_put_super(struct super_block *sb);
-#ifndef LINUX2_1
-void xfs_statfs(struct super_block *, struct statfs *, int);
-static int xfs_notify_change(struct inode *inode, struct iattr *attr);
-#else
 static void xfs_put_inode(struct inode *inode);
 static void xfs_write_super(struct super_block * sb);
-int xfs_statfs(struct super_block *, struct statfs *, int);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,51)
+static int xfs_statfs(struct super_block *sb, struct statfs *buf, int bufsiz);
+#else
+static int xfs_statfs(struct super_block *sb, struct statfs *buf);
+#endif
+
+#ifdef HAVE_STRUCT_SUPER_OPERATIONS_NOTIFY_CHANGE
 static int xfs_notify_change(struct dentry *dentry, struct iattr *attr);
-#endif /* LINUX2_1 */
-
-static struct inode *
-make_dead_vnode(struct super_block *sb);
-
-extern struct inode_operations xfs_dead_inode_operations;
+#endif
 
 static struct super_operations xfs_sops = { 
-#ifndef LINUX2_1
-    xfs_read_inode,	/* read_inode */
-    xfs_notify_change,	/* notify_change */
-    NULL,		/* write_inode */
-    xfs_delete_inode,	/* put_inode */
-    xfs_put_super,	/* put_super */
-    NULL,		/* write_super */
-    xfs_statfs,		/* xfs_statfs */
-    NULL		/* remount */
-#else
-    xfs_read_inode,	/* read_inode */
-    NULL,		/* write_inode */
-    xfs_put_inode,	/* put_inode */
-    xfs_delete_inode,	/* delete_inode */
-    xfs_notify_change,	/* notify_change */
-    xfs_put_super,	/* put_super */
-    xfs_write_super,	/* write_super */
-    xfs_statfs,		/* statfs */
-    NULL,		/* remount_fs */
-    NULL		/* clear_inode */
-#endif /* LINUX2_1 */
+    read_inode		: xfs_read_inode,
+    put_inode		: xfs_put_inode,
+    delete_inode	: xfs_delete_inode,
+#ifdef HAVE_STRUCT_SUPER_OPERATIONS_NOTIFY_CHANGE
+    notify_change	: xfs_notify_change,
+#endif
+    put_super		: xfs_put_super,
+    write_super		: xfs_write_super,
+    statfs		: xfs_statfs,
 };
 
+/*
+ * try to replace the fake root of `sb' with a real one
+ */
+
 int
-#ifndef LINUX2_1
-xfs_root(struct super_block *sb, struct inode **vpp)
-#else
-xfs_root(struct super_block *sb, struct dentry **dp)
-#endif /* LINUX2_1 */
+xfs_replace_root (struct super_block *sb)
 {
     struct xfs *xfsp = VFS_TO_XFS(sb);
-    struct xfs_message_getroot msg;
-    int error;
-    
-    XFSDEB(XDEBVFOPS, ("xfs_root sp = 0x%p\n", sb));
-    
-    do {
-	if (xfsp->root != NULL) {
-#ifndef LINUX2_1
-	    *vpp = XNODE_TO_VNODE(xfsp->root);
-#else
-#ifdef HAVE_D_ALLOC_ROOT_TWO_ARGS
-	    *dp = d_alloc_root(XNODE_TO_VNODE(xfsp->root), NULL);
-#else
-	    *dp = d_alloc_root(XNODE_TO_VNODE(xfsp->root));
-#endif
-	    xfs_d_init(*dp);
-	    DENTRY_TO_XDENTRY(*dp)->valid = 1;
-#endif /* LINUX2_1 */
-	    xfs[0].status |= XFS_ROOTINSTALLED;
-	    return 0;
-	}
-	
+    int error = 0;
+
+    XFSDEB(XDEBVFOPS, ("xfs_replace_root\n"));
+
+    while (xfsp->root == NULL && error == 0) {
+	struct xfs_message_getroot msg;
+
 	msg.header.opcode = XFS_MSG_GETROOT;
-	
 	/*
 	 * Mounting should done by root, so get the root node with
 	 * root's priviliges (usually none, and none is needed).
@@ -131,59 +94,105 @@ xfs_root(struct super_block *sb, struct dentry **dp)
 	msg.cred.pag = 0;
 	
 	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-	XFSDEB(XDEBVFOPS, ("xfs_root xfs_message_rpc error = %d\n", error));
+	XFSDEB(XDEBVFOPS,
+	       ("xfs_replace_root xfs_message_rpc error = %d\n", error));
 	if (error == 0)
 	    error = ((struct xfs_message_wakeup *) &msg)->error;
-	XFSDEB(XDEBVFOPS, ("xfs_root xfs_message_wakeup error = %d\n", error));
-    } while (error == 0);
-    
-    /*
-     * Failed to get message through, need to pretend that all went well
-     * and return a fake dead vnode to be able to unmount.
-     */
+	XFSDEB(XDEBVFOPS,
+	       ("xfs_replace_root xfs_message_wakeup error = %d\n", error));
+    }
+    if (error) {
+	XFSDEB(XDEBVFOPS, ("xfs_replace_root failed: %d\n", error));
+	return error;
+    }
 
-#ifndef LINUX2_1
-    *vpp = make_dead_vnode(sb);
-#else
-#ifdef HAVE_D_ALLOC_ROOT_TWO_ARGS
-    *dp = d_alloc_root(make_dead_vnode(sb), NULL);
-#else
-    *dp = d_alloc_root(make_dead_vnode(sb));
+    /* swap xfsp->root & sb->s_root->d_inode */
+    {
+	struct inode *inode = XNODE_TO_VNODE(xfsp->root);
+	struct inode *root  = sb->s_root->d_inode;
+	struct xfs_node *xn = VNODE_TO_XNODE(root);
+
+	XNODE_TO_VNODE(xfsp->root) = root;
+	VNODE_TO_XNODE(root)       = xfsp->root;
+	VNODE_TO_XNODE(inode)      = NULL;
+	XNODE_TO_VNODE(xn)         = inode;
+	free_xfs_node (xn);
+	iput (inode);
+
+	xfs_attr2inode (&xfsp->root->attr, root);
+	root->i_op = &xfs_dir_inode_operations;
+#ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
+	root->i_fop = &xfs_dir_operations;
 #endif
-    xfs_d_init(*dp);
-    DENTRY_TO_XDENTRY(*dp)->valid = 1;
-#endif /* LINUX2_1 */
-
+	xfsp->status |= XFS_ROOTINSTALLED;
+    }
+    XFSDEB(XDEBVFOPS, ("xfs_replace_root: success\n"));
     return 0;
 }
 
+/*
+ * create a `fake' root inode for `sb'
+ */
+
 static struct inode *
-make_dead_vnode(struct super_block *sb)
+make_root_vnode(struct super_block *sb)
 {
-    struct inode *dead;
-    
-    XFSDEB(XDEBNODE, ("make_dead_vnode sb = 0x%p\n", sb));
-    
-    dead = iget (sb,0);
-    if(!dead) { /* XXX - ugly.. better handling */
-	panic("make_dead_vnode: iget failed\n");
-    }
-    
-    dead->i_mode = 0555|S_IFDIR;
-    dead->i_op = &xfs_dead_inode_operations;
-    
-    return dead;
+    struct xfs_node *xn;
+    struct xfs_msg_node node;
+    struct inode *inode;
+
+    memset (&node, 0, sizeof(node));
+
+    node.handle.a = node.handle.b = node.handle.c = node.handle.d = 0;
+    node.anonrights = XFS_RIGHT_R;
+    XA_SET_MODE(&node.attr, S_IFDIR | 0777);
+    XA_SET_NLINK(&node.attr, 100);
+    XA_SET_SIZE(&node.attr, 0);
+    XA_SET_UID(&node.attr, 0);
+    XA_SET_GID(&node.attr, 0);
+    XA_SET_ATIME(&node.attr, 0);
+    XA_SET_MTIME(&node.attr, 0);
+    XA_SET_CTIME(&node.attr, 0);
+    XA_SET_FILEID(&node.attr, 0);
+    XA_SET_TYPE(&node.attr, XFS_FILE_DIR);
+
+    xn = new_xfs_node (VFS_TO_XFS(sb), &node);
+    inode = XNODE_TO_VNODE(xn);
+
+    inode->i_op = &xfs_dead_inode_operations;
+#ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
+    inode->i_fop = &xfs_dead_operations;
+#endif
+    XFS_TOKEN_CLEAR(xn, ~0,
+		    XFS_OPEN_MASK | XFS_ATTR_MASK | XFS_DATA_MASK | XFS_LOCK_MASK);
+    return inode;
 }
 
-#ifndef LINUX2_1
-struct super_block * xfs_read_super (struct super_block * sb, void * data,
-				      int silent)
+/*
+ * create a root dcache entry for `sb'
+ */
+
+static void
+make_root (struct super_block *sb)
+{
+    struct dentry *dp;
+
+#ifdef HAVE_D_ALLOC_ROOT_TWO_ARGS
+    dp = d_alloc_root(make_root_vnode(sb), NULL);
 #else
-struct super_block * xfs_read_super (struct super_block * sb, void * data,
-				      int silent, struct dentry *dir_d)
+    dp = d_alloc_root(make_root_vnode(sb));
 #endif
+    xfs_d_init(dp);
+    sb->s_root = dp;
+    xfs_replace_root (sb);
+}
+
+struct super_block *
+xfs_read_super (struct super_block * sb, void * data,
+		int silent)
 {
     int minordevice=0;
+    struct dentry *ddev;
     
     XFSDEB(XDEBVFOPS, ("xfs_read_super starting\n"));
     XFSDEB(XDEBVFOPS, ("xfs_read_super: sb: %p data: %p silent: %d\n",
@@ -193,8 +202,33 @@ struct super_block * xfs_read_super (struct super_block * sb, void * data,
 		       (int) MINOR(sb->s_dev)));
     
     XFSDEB(XDEBVFOPS, ("xfs_read_super: begin setting variables\n"));
+
+#ifndef LINUX2_3
     MOD_INC_USE_COUNT;
+#endif
     
+    if (data != NULL) {
+#ifndef LINUX2_3
+	ddev = lookup_dentry (data, NULL, 1);
+#else
+	{
+	    struct nameidata nd;
+	    int error = 0;
+
+	    if (path_init (data, LOOKUP_POSITIVE, &nd))
+		error = path_walk (data, &nd);
+	    if (error)
+		ddev = ERR_PTR(error);
+	    else
+		ddev = nd.dentry;
+	}
+#endif
+	if (!IS_ERR(ddev)) {
+	    minordevice = MINOR(ddev->d_inode->i_rdev);
+	    dput (ddev);
+	}
+    }
+
     xfs[minordevice].status |= XFS_MOUNTED;
     xfs[minordevice].sb = sb;
     xfs[minordevice].root = 0;
@@ -204,22 +238,7 @@ struct super_block * xfs_read_super (struct super_block * sb, void * data,
     VFS_TO_XFS(sb) = &xfs[minordevice];
     
     sb->s_op = &xfs_sops;
-#ifndef LINUX2_1
-    if (xfs_root(sb,&sb->s_mounted)) {
-	XFSDEB(XDEBVFOPS, ("xfs_read_super: get root inode failed\n"));
-	sb->s_dev=0;
-	MOD_DEC_USE_COUNT;
-	return NULL;
-    }
-#else
-    if (xfs_root(sb,&sb->s_root) || !sb->s_root) {
-	XFSDEB(XDEBVFOPS, ("xfs_read_super: get root inode failed\n"));
-	sb->s_dev=0;
-	MOD_DEC_USE_COUNT;
-	return NULL;
-    }
-#endif /* LINUX2_1 */
-
+    make_root(sb);
     sb->s_blocksize = 1024;
     sb->s_blocksize_bits = 10;
     
@@ -228,26 +247,30 @@ struct super_block * xfs_read_super (struct super_block * sb, void * data,
     return sb;
 }
 
-#ifdef LINUX2_1
-static void xfs_write_super(struct super_block * sb)
+static void
+xfs_write_super(struct super_block * sb)
 {
     sb->s_dirt = 0;
 }
-#endif /* LINUX2_1 */
 
-static void xfs_put_super(struct super_block *sb)
+static void
+xfs_put_super(struct super_block *sb)
 {
-    int minordevice = 0;
+    struct xfs *xfsp = VFS_TO_XFS(sb);
+
     XFSDEB(XDEBVFOPS, ("xfs_put_super starting\n"));
-    xfs[minordevice].status &= ~XFS_MOUNTED;
-    xfs[minordevice].status &= ~XFS_ROOTINSTALLED;
+    xfsp->status &= ~XFS_MOUNTED;
+    xfsp->status &= ~XFS_ROOTINSTALLED;
     sb->s_dev = 0;
-    xfs[minordevice].root = NULL;
+    xfsp->root = NULL;
     XFSDEB(XDEBVFOPS, ("xfs_put_super exiting\n"));
+#ifndef LINUX2_3
     MOD_DEC_USE_COUNT;
+#endif
 }
 
-static void xfs_read_inode(struct inode *inode)
+static void
+xfs_read_inode(struct inode *inode)
 {
     XFSDEB(XDEBVFOPS,("xfs_read_inode starting inode: %p\n",inode));
     XFSDEB(XDEBVFOPS,("xfs_read_inode inode->i_ino: %ld\n",inode->i_ino));
@@ -256,46 +279,58 @@ static void xfs_read_inode(struct inode *inode)
     inode->i_op = NULL;
     XFSDEB(XDEBVFOPS,("xfs_read_inode exiting\n"));
 }
+ 
+/*
+ * Called `inode' is de-referenced, ie when iput is called.
+ */
 
-#ifdef LINUX2_1
-static void xfs_put_inode(struct inode *inode)
+static void
+xfs_put_inode(struct inode *inode)
 {
     XFSDEB(XDEBVFOPS,("xfs_put_inode: inode: %p count: %d aliases:",
-		      inode, inode->i_count));
+		      inode, xfs_icount(inode)));
     print_aliases(inode);
-    if (inode->i_count == 1)
-	inode->i_nlink = 0;
-}
-#endif
 
-static void xfs_delete_inode(struct inode *inode)
+    if (xfs_icount(inode) == 1) {
+	struct xfs_node *xn = VNODE_TO_XNODE(inode);
+
+	inode->i_nlink = 0;
+
+	if (xn) {
+	    if (xn->flags & XFS_STALE)
+		xfs_force_invalid_xnode (xn);
+	    xn->flags &= ~XFS_STALE;
+	}
+    }
+}
+
+/*
+ * `inode' is about to die.  add it to the list of nodes to be inactivated.
+ * this is not done immediately to avoid allocating memory in a function
+ * that's called when memory is scarse.
+ */
+
+static void
+xfs_delete_inode(struct inode *inode)
 {
-    struct xfs_message_inactivenode msg;
-    struct xfs *xfsp = XFS_FROM_VNODE(inode);
     struct xfs_node *xn = VNODE_TO_XNODE(inode);
 
     XFSDEB(XDEBNODE,
 	   ("xfs_delete_inode inode = 0x%p i_ino:0x%lx i_count:%u\n",
-	    inode, inode->i_ino, inode->i_count));
+	    inode, inode->i_ino, xfs_icount(inode)));
 
-    if (xn == NULL)
+    if (xn == NULL) {
+	clear_inode (inode);
 	return;
+    }
 
-    msg.header.opcode = XFS_MSG_INACTIVENODE;
-    msg.handle = xn->handle;
-    msg.flag   = XFS_NOREFS | XFS_DELETE;
-    xfs_message_send(xfsp->fd, &msg.header, sizeof(msg));
-
-#ifndef LINUX2_1
-    xfs_cache_delete(xn);
-#endif /* LINUX2_1 */
-
-    free_xfs_node(xn);
+    clear_xfs_node (xn);
+    xfs_queue_inactive (xn);
     clear_inode(inode);
 }
 
 void
-vattr2xfs_attr(struct iattr *iattr, struct xfs_attr *attr)
+vattr2xfs_attr(const struct iattr *iattr, struct xfs_attr *attr)
 {
     XA_CLEAR(attr);
     if (iattr->ia_valid & ATTR_MODE)
@@ -314,16 +349,11 @@ vattr2xfs_attr(struct iattr *iattr, struct xfs_attr *attr)
 	XA_SET_SIZE(attr,iattr->ia_size);
 }
 
+#ifdef HAVE_STRUCT_SUPER_OPERATIONS_NOTIFY_CHANGE
 static int
-#ifndef LINUX2_1
-xfs_notify_change(struct inode *inode, struct iattr *attr)
-#else
 xfs_notify_change(struct dentry *dentry, struct iattr *attr)
-#endif
 {
-#ifdef LINUX2_1
     struct inode *inode = dentry->d_inode;
-#endif
     struct xfs *xfsp = XFS_FROM_VNODE(inode);
     struct xfs_node *xn = VNODE_TO_XNODE(inode);
     int error = 0;
@@ -343,39 +373,49 @@ xfs_notify_change(struct dentry *dentry, struct iattr *attr)
         msg.handle = xn->handle;
         vattr2xfs_attr(attr, &msg.attr);
 	inode_setattr(inode, attr);
+	if (XFS_TOKEN_GOT(xn, XFS_DATA_R)) {
+	    if (S_ISREG(inode->i_mode))
+		XA_SET_SIZE(&msg.attr,  inode->i_size);
+	    XA_SET_MTIME(&msg.attr, inode->i_mtime);
+	}
+        XFS_TOKEN_CLEAR(xn, XFS_ATTR_VALID, XFS_ATTR_MASK);
         error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
         if (error == 0) {
             error = ((struct xfs_message_wakeup *) & msg)->error;
 	    
-#ifndef LINUX2_1
-	    inode->i_dirt = 0;
-#endif /* LINUX2_1 */
 	}
-        XFS_TOKEN_CLEAR(xn, XFS_ATTR_VALID, XFS_ATTR_MASK);
     }
     
     return error;
 }
+#endif /* HAVE_STRUCT_SUPER_OPERATIONS_NOTIFY_CHANGE */
 
-#ifndef LINUX2_1
-void
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,51)
+static int xfs_statfs(struct super_block *sb, struct statfs *buf, int bufsiz)
 #else
-int
-#endif /* LINUX2_1 */
-xfs_statfs(struct super_block *sb, struct statfs *buf, int bufsiz)
+static int xfs_statfs(struct super_block *sb, struct statfs *buf)
+#endif
 {
     struct statfs tmp;
     
-    tmp.f_type = 0x47114711;
-    tmp.f_bsize = sb->s_blocksize;
-    tmp.f_blocks = 1024*1024;
-    tmp.f_bfree = 1024*1024;
-    tmp.f_bavail = 1024*1024;
-    tmp.f_files = 0;
-    tmp.f_ffree = 1024*1024;
+    tmp.f_type    = 0x47114711;
+    tmp.f_bsize   = sb->s_blocksize;
+    tmp.f_blocks  = 1024*1024;
+    tmp.f_bfree   = 1024*1024;
+    tmp.f_bavail  = 1024*1024;
+    tmp.f_files   = 0;
+    tmp.f_ffree   = 1024*1024;
     tmp.f_namelen = NAME_MAX;
-#ifdef LINUX2_1
-    return
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,51)
+    return copy_to_user(buf, &tmp, bufsiz) ? -EFAULT : 0;
+#else
+    *buf = tmp;
+    return 0;
 #endif
-    copy_to_user(buf, &tmp, bufsiz) ? -EFAULT : 0;
+}
+
+void
+print_nodes(int i)
+{
+    printk("%d nodes\n", xfs[i].nnodes);
 }
