@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls.c,v 1.22 1999/08/02 11:43:57 assar Exp $");
+RCSID("$Id: xfs_syscalls.c,v 1.29 2000/10/02 23:53:27 lha Exp $");
 
 /*
  * XFS system calls.
@@ -83,7 +78,7 @@ xfs_is_pag(struct cred *cred)
 	return 0;
 }
 
-pag_t
+xfs_pag_t
 xfs_get_pag(struct cred * cred)
 {
     if (xfs_is_pag(cred)) {
@@ -150,7 +145,7 @@ xfs_setpag_call(struct proc *proc)
 
 static struct sysent old_setgroups;
 
-static int (*old_setgroups_func)(u_int, gid_t *);
+static int (*old_setgroups_func)(u_int, gid_t *) = NULL;
 
 static int xfs_setgroups (u_int, gid_t *);
 
@@ -165,7 +160,10 @@ xfs_install_setgroups(void)
 void
 xfs_uninstall_setgroups(void)
 {
-    sysent[SYS_setgroups] = old_setgroups;
+    if (old_setgroups_func) {
+	sysent[SYS_setgroups] = old_setgroups;
+	old_setgroups_func = NULL;
+    }
 }
 
 /*
@@ -192,32 +190,44 @@ xfs_unpag (struct cred *cred)
 static int
 xfs_setgroups (u_int gidsetsize, gid_t *gidset)
 {
-    if (xfs_is_pag (curproc->p_cred)) {
+    proc_t *p = ttoproc(curthread);
+
+    mutex_enter(&p->p_crlock);
+
+    if (xfs_is_pag (p->p_cred)) {
 	gid_t part1, part2;
 	int ret;
 
-	if (gidsetsize + 2 >= ngroups_max)
+	if (gidsetsize + 2 >= ngroups_max) {
+	    mutex_exit(&p->p_crlock);
 	    return set_errno(E2BIG);
+	}
 
-	part1 = curproc->p_cred->cr_groups[1];
-	part2 = curproc->p_cred->cr_groups[2];
+	part1 = p->p_cred->cr_groups[1];
+	part2 = p->p_cred->cr_groups[2];
 	ret   = (*old_setgroups_func) (gidsetsize, gidset);
-	if (ret)
+	if (ret) {
+	    mutex_exit(&p->p_crlock);
 	    return set_errno(ret);
-	ret = store_pag (&curproc->p_cred, part1, part2);
-	if (ret)
+	}
+	ret = store_pag (&p->p_cred, part1, part2);
+	if (ret) {
+	    mutex_exit(&p->p_crlock);
 	    return set_errno(ret);
-	return 0;
+	}
     } else {
 	int ret;
 
 	ret = (*old_setgroups_func) (gidsetsize, gidset);
-	if (xfs_is_pag (curproc->p_cred))
-	    xfs_unpag (curproc->p_cred);
-	if (ret)
+	if (xfs_is_pag (p->p_cred))
+	    xfs_unpag (p->p_cred);
+	if (ret) {
+	    mutex_exit(&p->p_crlock);
 	    return set_errno(ret);
-	return 0;
+	}
     }
+    mutex_exit(&p->p_crlock);
+    return 0;
 }
 
 /*
@@ -336,8 +346,8 @@ remote_pioctl (int a_opcode,
 
     if (error == 0)
 	error = msg2->error;
-    else
-	error = EINVAL; /* return EINVAL to not confuse applications */
+    if (error == ENODEV || error == ENXIO)
+	error = EINVAL;
     
     if (error == 0 && vice_ioctl->out_size)
 	error = copyout((caddr_t)msg2->msg,
@@ -385,6 +395,41 @@ xfs_debug(struct ViceIoctl *vice_ioctl)
     return 0;
 }
 
+/*
+ * print interesting debug information
+ */
+
+static int
+xfs_debug_print (struct ViceIoctl *vice_ioctl, struct vnode *vp)
+{
+    int32_t flags;
+    int error;
+
+    if (!suser(CRED()))
+	return EPERM;
+
+    if (vice_ioctl->in_size != 0) {
+	int32_t tmp;
+
+	if (vice_ioctl->in_size < sizeof(int32_t))
+	    return EINVAL;
+	
+	error = copyin (vice_ioctl->in,
+			&tmp,
+			sizeof(tmp));
+	if (error)
+	    return error;
+	switch (tmp) {
+	case XDEBNODE :
+	    xfs_uprintf_filsys ();
+	    break;
+	default :
+	    return EINVAL;
+	}
+    }
+    return 0;
+}
+
 /*
  * Handle `pioctl'
  */
@@ -424,16 +469,27 @@ xfs_pioctl_call(char *a_pathP,
 
     switch (a_opcode) {
     case VIOC_FHGET :
+#ifdef VIOC_FHGET_32
     case VIOC_FHGET_32 :
+#endif
 	XFSDEB(XDEBSYS, ("calling fhget(%x, %x)\n",
 			 (int)vp, (int)a_paramsP));
 	return fhget_call (vp, a_paramsP);
     case VIOC_FHOPEN :
+#ifdef VIOC_FHOPEN_32
     case VIOC_FHOPEN_32 :
+#endif
 	return fhopen_call (vp, a_paramsP, a_followSymlinks);
     case VIOC_XFSDEBUG :
+#ifdef VIOC_XFSDEBUG_32
     case VIOC_XFSDEBUG_32 :
+#endif
 	return xfs_debug (a_paramsP);
+    case VIOC_XFSDEBUG_PRINT :
+#ifdef VIOC_XFSDEBUG_PRINT_32
+    case VIOC_XFSDEBUG_PRINT_32 :
+#endif
+	return xfs_debug_print (a_paramsP, vp);
     default :
 	return remote_pioctl (a_opcode, vp, a_paramsP);
     }
